#!/usr/bin/perl -w

#
# vCenter Discovery and Information Gathering
#
# Gather information from vCenter, Hosts and VMs.
#

package VCD;
use Exporter;
use Data::UUID;
#use JSON; #@see listVMFS

@ISA = ('Exporter');
@EXPORT = (
   'listDC',
   'listClusters',
   'listHosts',
   'listVMFS',
   'serviceGetSfcbdWatchdogStatus',
   'serviceSfcbdWatchdogRestart',
   'listHostsVMs',
   'getMaintenanceMode',
   'hostInfo',
   'vmInfo',
   'vdiskInfo',
   'getvCenterVersion',
   'accessHostAnnotation',
   'accessVMAnnotation',
   'accessDataCenterAnnotation',
   'getAnnotatedVMs',
   'getAnnotatedHosts',
   'generateUUID',
   'enableDiskUUID'
);

use strict;
use warnings;
use Switch;

use VMware::VIRuntime;
use VMware::VILib;
use VMware::VICredStore;
use constant ANNOTATED_BY_ANOTHER_CONSOLE	=> 0;
use constant ANNOTATED_BY_THIS_CONSOLE		=> 1;
use constant NOT_ANNOTATED          		=> 2;

sub VCD::listDC()
{
   my $datacenter_view = Vim::find_entity_views(view_type => 'Datacenter',
                                                properties => [ "name" ] );
   foreach (@$datacenter_view) 
   {
      print $_->name ."\n";
   }
}

sub VCD::listClusters()
{
   my $cluster_view = Vim::find_entity_views(view_type => 'ClusterComputeResource',
					     properties => [ "name" ] );
   foreach (@$cluster_view) 
   {
      print $_->name ."\n";
   }
}


sub VCD::get_entities 
{
   my %args = @_;
   my $view_type = $args{view_type};
   my $obj = $args{obj};

   my $sc = Vim::get_service_content();
   my $service = Vim::get_vim_service();

   my $property_spec = PropertySpec->new(all => 0,
                                         type => $view_type->get_backing_type());
   my $property_filter_spec = $view_type->get_search_filter_spec($obj,[$property_spec]);
   my $retrieve_options = RetrieveOptions->new(); # Default = all results.
   my $obj_contents = $service->RetrievePropertiesEx(_this => $sc->propertyCollector,
                                                   specSet => $property_filter_spec, options => $retrieve_options);
   my $result = Util::check_fault($obj_contents);

   # Currently only a basic conversion to RetrievePropertiesEx has been
   # implemented because RetrieveProperties is deprecated.
   # Eventually we could use its full facilities for batching in results,
   # but for now returning the whole set at once is safest (same as before).
   if (defined($result) && defined($result->objects)) {
     return $result->objects;
   }

   # No results. For consistency with the old version, return an empty array.
   my @empty_array;
   return \@empty_array;
}


#
# List all the hosts managed by a vcenter
#
sub VCD::listHosts($$)
{
    my ($dcname, $clusname) = @_;

    my $hostlist;
    my $datacenter;
    my $cluster;

    if (defined $dcname) {
        my $datacenter_views = Vim::find_entity_views (view_type => 'Datacenter',
                                                       filter => {name => $dcname});
        die "Datacenter '$dcname' not found" unless (@$datacenter_views);

        foreach(@$datacenter_views) {
            $datacenter = $_;
        }

        my $result = get_entities(view_type => 'HostSystem',
                                  obj => $datacenter);
        foreach (@$result) {
            my $obj_content = $_;
            my $mob = $obj_content->obj;
            my $obj = Vim::get_view(mo_ref=>$mob,
                                    properties => [ "name" ] );
            print "Host : " . $obj->name . "\n";
        }

    } elsif (defined $clusname) {
	my $cluster_views = Vim::find_entity_views (view_type => 'ClusterComputeResource',
						    filter => {name => $clusname});
	die "Cluster '$clusname' not found" unless (@$cluster_views);

	foreach(@$cluster_views) {
	    $cluster = $_;
	}

        my $result = get_entities(view_type => 'HostSystem',
                                  obj => $cluster);
        foreach (@$result) {
            my $obj_content = $_;
            my $mob = $obj_content->obj;
            my $obj = Vim::get_view(mo_ref=>$mob,
                                    properties => [ "name" ] );
            print "Host : " . $obj->name . "\n";
        }

    } else {
        eval {
            $hostlist = Vim::find_entity_views(view_type => 'HostSystem',
                                               properties => [ "summary" ] );
        };
        die $@ if ($@);
        foreach (@$hostlist) {
            print "Host : " . $_->summary->config->name . "\n";
        }
    }
}

#
# Get the cluster a VM belongs to
# Args:
#      $vm - Virtual machine
#
sub VCD::vmHostList($$) {
    my ($vmname, $type) = @_;

    my $vtype;
    $vtype = "ClusterComputeResource" if ((defined $type) && ($type eq "cluster"));
    $vtype = "Datacenter" if ((defined $type) && ($type eq "datacenter"));
    return if (! defined $vtype);
    my $views = Vim::find_entity_views (view_type => $vtype);
    foreach(@$views) {
	my $parent_obj = $_;
        my $result = get_entities(view_type => 'VirtualMachine', obj => $parent_obj);
        foreach (@$result) {
            my $obj_content = $_;
            my $mob = $obj_content->obj;
            my $obj = Vim::get_view(mo_ref=>$mob,
                                    properties => [ "name" ] );
	    if ($obj->name eq $vmname) {
		# Found the parent object, now find the hosts
		my $hresult = get_entities(view_type => 'HostSystem',  obj => $parent_obj);
		foreach (@$hresult) {
		    my $obj_content = $_;
		    my $mob = $obj_content->obj;
		    my $obj = Vim::get_view(mo_ref=>$mob,  properties => [ "name" ] );
		    print "Host : " . $obj->name . "\n";
		}
		return;
	    }
	}
    }
}


#
# List all the hosts and their datastores managed by a vcenter
#
sub VCD::listVMFS($$)
{
    my ($dcname, $clusname) = @_;

    my $hostlist;
    my $datacenter;
    my $cluster;

    if (defined $dcname) {
      # Note: This is the only block that is currently used.
      
      my $hashresult = {};

      my $datacenter_views = Vim::find_entity_views (view_type => 'Datacenter',
                                                     filter => {name => $dcname});
      die "Datacenter '$dcname' not found" unless (@$datacenter_views);
      $datacenter = $$datacenter_views[0];

      my $result = get_entities(view_type => 'HostSystem',
                                obj => $datacenter);
      foreach (@$result) {
        my $obj_content = $_;
        my $mob = $obj_content->obj;
        my $obj = Vim::get_view(mo_ref=>$mob,
                                properties => [ "name", "datastore" ] );

        my $datastores;
        eval {
          $datastores = Vim::get_views(mo_ref_array => $obj->datastore,
                 properties => [ "summary", "info" ]);
        };

        foreach(@$datastores) {
          if ($_->summary->type eq 'VMFS') {
            # Build a hash of datastores with result information about them.
            if (! $hashresult->{$_->summary->name}) {
              $hashresult->{$_->summary->name}->{'vmfs_version'} = $_->info->vmfs->majorVersion;
              $hashresult->{$_->summary->name}->{'hosts'} = [];
            }
            push(@{$hashresult->{$_->summary->name}->{'hosts'}}, $obj->name);
          }
        }
      }

      # TODO: JSON is the preferred way to transmit the data, but
      # we are trying to avoid installing more open source libraries in Copper.
      # Note that this line below does not wrap the result in JSend wrappers,
      # so need to do that before we can use it.
      #print JSON->new->utf8->encode($hashresult) . "\n\n";

      # In the meantime, manually output JSON.  This is dumb but only used in
      # a very limited / safe context, so it will do until Dubnium.
      print '{"data":{';
      my $is_first = 1;
      for (keys %$hashresult) {
        if ($is_first eq 1) {
          $is_first = 0;
        } else {
          print ',';
        }

        my $datastore_name = $_;
        $datastore_name =~ s/\"/\\\"/g; # only field that may contain a "
        my $vmfs_version = $hashresult->{$datastore_name}->{'vmfs_version'};
        my $hosts = join('","', @{$hashresult->{$datastore_name}->{'hosts'}});
        print "\"$datastore_name\":{\"hosts\":[\"$hosts\"],\"vmfs_version\":\"$vmfs_version\"}";
      }
      print '},"status":"success"}';

    } elsif (defined $clusname) {
      die 'Error: listVMFS with $clusname is untested. Test and ensure equivalence before use.';
	my $cluster_views = Vim::find_entity_views (view_type => 'ClusterComputeResource',
						    filter => {name => $clusname});
	die "Cluster '$clusname' not found" unless (@$cluster_views);
	$cluster = $$cluster_views[0];

        my $result = get_entities(view_type => 'HostSystem',
                                  obj => $cluster);
        foreach (@$result) {
            my $obj_content = $_;
            my $mob = $obj_content->obj;
            my $obj = Vim::get_view(mo_ref=>$mob,
                                    properties => [ "name", "datastore" ] );
            print "Host: " . $obj->name . "\n";
	    my $datastores;
	    eval {
		$datastores = Vim::get_views(mo_ref_array => $obj->datastore,
					     properties => [ "summary" ]);
	    };
	    foreach(@$datastores) {
		if ($_->summary->type eq 'VMFS') {
		    print "Datastore: " . $_->summary->name . "\n";
		}
	    }
        }

    } else {
      die 'Error: listVMFS "else" is untested. Test and ensure equivalence before use.';
        eval {
            $hostlist = Vim::find_entity_views(view_type => 'HostSystem',
                                               properties => [ "summary", "datastore" ] );
        };
        die $@ if ($@);
        foreach (@$hostlist) {
            print "Host: " . $_->summary->config->name . "\n";
	    my $datastores;
	    eval {
		$datastores = Vim::get_views(mo_ref_array => $_->datastore,
					     properties => [ "summary" ]);
	    };
	    foreach(@$datastores) {
		if ($_->summary->type eq 'VMFS') {
		    print "  Datastore: " . $_->summary->name . "\n";
		}
	    }
        }
    }
}


# Get sfcbd-watchdog status
# Args:
#       $host - Host system,
#
# Returns:
#       nothing
sub VCD::serviceGetSfcbdWatchdogStatus {
        my ($hostname) = @_;
        my $sfcbdWatchdogStatus = PrimType->new(0, 'boolean');
        my $hostServiceSystem;
        my $serviceSystem;

        my $host = Vim::find_entity_view(
                view_type => 'HostSystem',
                filter => { name => $hostname },
                properties => ["configManager"],
                );

        eval {
                $hostServiceSystem = Vim::get_view(mo_ref => $host->configManager->serviceSystem);
        };
        if ($@) {
                die "Error: Could not connect to host";
        }
        
        eval { 
                $hostServiceSystem->RefreshServices();
        };
        if ($@) {
                die "Error: Could not refresh services";
        }
        
        eval {
                $serviceSystem = Vim::get_view(mo_ref => $host->configManager->serviceSystem);
        };
        if ($@) {
                die "Error: Could not connect to host";
        }

        my $services = $serviceSystem->serviceInfo->service;

        foreach my $service (@$services) {
                my $key = $service->key;
                if ($key eq "sfcbd-watchdog") {
                        if ($service->running eq 1) {
                                print "SFCBD Status:\tStarted\n";
                        } else {
                                print "SFCBD Status:\tStopped\n";
                        }
                        return;
                }
        }
}       

#
# Restart the sfcbd-watchdog
# Args:
#   $host - Host system,
#
# Returns:
#   nothing
sub VCD::serviceSfcbdWatchdogRestart {
    my ($hostname) = @_;

    my $host = Vim::find_entity_view(
                            view_type => 'HostSystem',
                            filter => { name => $hostname },
                            properties => ["configManager"],
                            );

    my $hostServiceSystem = Vim::get_view(mo_ref => $host->configManager->serviceSystem);
    die "Error: Host service system not found" unless ($hostServiceSystem);

    my $max_tries = 4;
    for (my $tries = 1; $tries <= $max_tries; $tries++) {
        eval {
            $hostServiceSystem->RestartService(id => "sfcbd-watchdog");
        };
        if (!$@) { last; }

        my $error = $@;

        # burt 720633: allow a few occurences of this specific error without
        # dying; instead retry the restart.
        if (($tries < $max_tries) &&
            ($error->name =~ /SystemErrorFault/) &&
            ($error->fault_string =~ /A general system error occurred: Invalid fault/)) {
            next;
        }
        die "Error: " . $error->name. '. '. $error->fault_string;
    }
    
    print "Sfcbd-watchdog restarted\n";
}

#
# List all the Hosts and VMs managed by a vcenter
#
sub VCD::listHostsVMs($$)
{
    my ($dcname, $clusname) = @_;

    my $hostlist;
    my $datacenter;
    my $cluster;

    if (defined $dcname) {
        my $datacenter_views = Vim::find_entity_views (view_type => 'Datacenter',
                                                       filter => {name => $dcname});
        die "Datacenter '$dcname' not found" unless (@$datacenter_views);

        foreach(@$datacenter_views) {
            $datacenter = $_;
        }

        my $result = get_entities(view_type => 'HostSystem',
                                  obj => $datacenter);
        foreach (@$result) {
            my $obj_content = $_;
            my $mob = $obj_content->obj;
            my $obj = Vim::get_view(mo_ref=>$mob,
                                    properties => [ "name", "vm" ] );
            print "Host : " . $obj->name . "\n";
            my $vms;
            eval {
                $vms =  Vim::get_views(mo_ref_array => $obj->vm,
                                       properties => [ "summary" ]);
            };
            die $@ if ($@);
            foreach (@$vms) {
		next if ($_->summary->config->template);
                print "   VM : " . $_->summary->config->name . "\n";
            }
        }

    } elsif (defined $clusname) {

	my $cluster_views = Vim::find_entity_views (view_type => 'ClusterComputeResource',
						    filter => {name => $clusname});
	die "Cluster '$clusname' not found" unless (@$cluster_views);

	foreach(@$cluster_views) {
	    $cluster = $_;
	}

        my $result = get_entities(view_type => 'HostSystem',
                                  obj => $cluster);
        foreach (@$result) {
            my $obj_content = $_;
            my $mob = $obj_content->obj;
            my $obj = Vim::get_view(mo_ref=>$mob,
                                    properties => [ "name", "vm" ] );
            print "Host : " . $obj->name . "\n";
            my $vms;
            eval {
                $vms =  Vim::get_views(mo_ref_array => $obj->vm,
                                       properties => [ "summary" ]);
            };
            die $@ if ($@);
            foreach (@$vms) {
		next if ($_->summary->config->template);
                print "   VM : " . $_->summary->config->name . "\n";
            }
        }

    } else {

        eval {
            $hostlist = Vim::find_entity_views(view_type => 'HostSystem',
                                               properties => [ "summary", "vm" ] );
        };
        die $@ if ($@);
        foreach (@$hostlist) {
            print "Host : " . $_->summary->config->name . "\n";
            my $vms;
            eval {
                $vms =  Vim::get_views(mo_ref_array => $_->vm,
                                       properties => [ "summary" ]);
            };
            die $@ if ($@);
            foreach (@$vms) {
		next if ($_->summary->config->template);
                print "   VM : " . $_->summary->config->name . "\n";
            }
        }
    }
}

#
# Check to see if host is in maintenance mode
sub VCD::getMaintenanceMode($)
{
    my ($hostname) = @_;
    
    my $hostlist;
    die "No hostname specified" unless ($hostname);
    eval {
        $hostlist = Vim::find_entity_views(view_type => 'HostSystem',
                                           filter => { name => $hostname } ,
                                           properties => [ "runtime" ] );
    };
    die $@ if ($@);
    foreach (@$hostlist) {
        my $mode = $_->runtime->inMaintenanceMode;
        print "Maintenance mode: $mode\n";
    }
}

#
# Print information about the host
#
sub VCD::hostInfo($)
{
    my ($hostname) = @_;

    my $hostlist;
    die "No hostname specified" unless ($hostname);

    my %host_status = (
                       'gray' => 'The status is unknown',
                       'green' => 'The entity is OK',
                       'red' => 'The entity definitely has a problem',
                       'yellow' => 'The entity might have a problem',
                       );

    eval {
        $hostlist = Vim::find_entity_views(view_type => 'HostSystem',
                                           filter => { name => $hostname } ,
                                           properties => [ "name", "summary", "runtime", "hardware" ] );
    };
    die $@ if ($@);
    foreach (@$hostlist) {
        my $status = $_->summary->overallStatus->val;
        my $host_view = $_;

        print "Host Name:\t" . $host_view->name . "\n";
        print "Host Status:\t" . $host_status{$status} . "\n";

        if (defined ($host_view->runtime->bootTime)) {
            print "BootTime:\t" . $host_view->runtime->bootTime . "\n";
        } else {
            print "BootTime:\tNot Known\n";
        }
        if (defined ($host_view->summary->hardware)) {
            print "Cpu Model:\t" . $host_view->summary->hardware->cpuModel . "\n";
        } else {
            print "Cpu Model:\tNot Known\n";
        }
        if (defined ($host_view->hardware)) {
            print "Cpu Speed:\t" . $host_view->hardware->cpuInfo->hz * $host_view->hardware->cpuInfo->numCpuCores . "\n";
        } else {
            print "Cpu Speed:\tNot Known\n";
        }
        if (defined ($host_view->summary->quickStats->overallCpuUsage)) {
            print "Cpu Usage:\t" . $host_view->summary->quickStats->overallCpuUsage . "\n";
        } else {
            print "Cpu Usage:\tNot Known\n";
        }
        if (defined ($host_view->summary->quickStats->overallMemoryUsage)) {
            print "Memory Usage:\t" . $host_view->summary->quickStats->overallMemoryUsage . "\n";
        } else {
            print "Memory Usage:\tNot Known\n";
        }
        if (defined ($host_view->summary->rebootRequired)) {
            print "Reboot Required:\t" . $host_view->summary->rebootRequired . "\n";
        } else {
            print "Reboot Required:\tNot Known\n";
        }
        if (defined ($host_view->runtime->inMaintenanceMode)) {
            print "Maintenance mode:\t" . $host_view->runtime->inMaintenanceMode . "\n";
        } else {
            print "Maintenance mode:\tNot Known\n";
        }
        my $pendingAlarms = "No";
        eval {
            my $alarmState = $host_view->triggeredAlarmState();
            foreach (@$alarmState) {
                if ($_->overallStatus->val eq "red" || $_->overallStatus->val eq "yellow") {
                    $pendingAlarms = "Yes";
                }
            }
        };
        print "Pending Alarms:\t" . $pendingAlarms. "\n";
        print "Model:\t" . $host_view->hardware->systemInfo->model . "\n";
        print "Vendor:\t" . $host_view->hardware->systemInfo->vendor . "\n";
        print "VMotion:\t" . $host_view->summary->config->vmotionEnabled . "\n";
        print "Physical Memory:\t" . $host_view->summary->hardware->memorySize . " bytes\n";

        if (defined ($host_view->summary->config->product)) {
            print "Software on host:\t" . ${$host_view->summary->config->product}{'fullName'} . "\n";
        }

	# Check cluster membership and whether DRS and HA are enabled
	my $cluster_views = Vim::find_entity_views(view_type => 'ClusterComputeResource');
	foreach (@$cluster_views) {
	    my $cluster = $_;
	    my $result = get_entities(view_type => 'HostSystem',
				      obj => $cluster);
	    foreach (@$result) {
		my $obj_content = $_;
		my $mob = $obj_content->obj;
		my $obj = Vim::get_view(mo_ref=>$mob,
					properties => [ "name" ] );
		if ($obj->name eq $host_view->name) {
		    print "Cluster:\t" . $cluster->name . "\n";
		    print "DRS:\t" . $cluster->configuration->drsConfig->enabled . "\n";
		    print "HA:\t" . $cluster->configuration->dasConfig->enabled . "\n";
		    return;
		}
	    }
        }
	# This host is not part of a cluster
	print "DRS:\t0\n";
	print "HA:\t0\n";
    }
}


#
# Print information about the VM
#
sub VCD::vmInfo($)
{
    my ($vmname) = @_;

    my %toolsStatus = (
                       'toolsNotInstalled' => 'VMware Tools has never been installed or has '
                       .'not run in the virtual machine.',
                       'toolsNotRunning' => 'VMware Tools is not running.',
                       'toolsOk' => 'VMware Tools is running and the version is current',
                       'toolsOld' => 'VMware Tools is running, but the version is not current',
                       );

    my $vmlist;
    die "No VM name specified" unless ($vmname);
    eval {
        $vmlist = Vim::find_entity_views(view_type => 'VirtualMachine',
                                         filter => { name => $vmname } ,
                                         properties => [ "config", "summary", "guest" ] );
    };
    die $@ if ($@);

    foreach (@$vmlist) {
        my $vm_view = $_;

        if (defined($vm_view->config) && defined ($vm_view->config->name)) {
            print "Name:\t" . $vm_view->config->name . "\n";
        } else {
            print "Name:\tNot Known\n";
        }
        if (defined ($vm_view->summary->runtime->host)) {
            my $host = Vim::get_view(mo_ref => $vm_view->summary->runtime->host,
                                properties => [ "name" ]);
            print "Current Host:\t" . $host->name . "\n";
        } else {
            print "Current Host:\tNot Known\n";
        }
        if (defined ($vm_view->summary->config->numCpu)) {
            print "No. of CPU(s):\t" . $vm_view->summary->config->numCpu . "\n";
        } else {
            print "No. of CPU(s):\tNot Known\n";
        }
        if (defined ($vm_view->summary->config->memorySizeMB)) {
            print "Memory Size:\t" . $vm_view->summary->config->memorySizeMB . "\n";
        } else {
            print "Memory Size:\tNot Known\n";
        }
        if (defined ($vm_view->summary->config->numVirtualDisks)) {
            print "Virtual Disks:\t" . $vm_view->summary->config->numVirtualDisks . "\n";
        } else {
            print "Virtual Disks:\tNot Known\n";
        }
        if (defined ($vm_view->summary->config->template)) {
            print "Template:\t" . $vm_view->summary->config->template . "\n";
        } else {
            print "Template:\tNot Known\n";
        }
        if (defined ($vm_view->summary->config->vmPathName)) {
            print "vmPathName:\t" . $vm_view->summary->config->vmPathName . "\n";
        } else {
            print "vmPathName:\tNot Known\n";
        }
        if (defined ($vm_view->summary->runtime->powerState->val)) {
            print "Power State:\t" . $vm_view->summary->runtime->powerState->val . "\n";
        } else {
            print "Power State:\tNot Known\n";
        }
        if (defined ($vm_view->summary->guest->guestFullName)) {
            print "Guest OS:\t" . $vm_view->summary->guest->guestFullName . "\n";
        } else {
            print "Guest OS:\tNot Known\n";
        }
        if (defined ($vm_view->summary->guest->guestId)) {
            print "guestId:\t" . $vm_view->summary->guest->guestId . "\n";
        } else {
            print "guestId:\tNot Known\n";
        }
        if (defined ($vm_view->summary->guest->hostName)) {
            print "Host name:\t" . $vm_view->summary->guest->hostName . "\n";
        } else {
            print "Host name:\tNot Known\n";
        }
        if (defined ($vm_view->summary->guest->ipAddress)) {
            print "IP Address:\t" . $vm_view->summary->guest->ipAddress . "\n";
        } else {
            print "IP Address:\tNot Known\n";
        }

        if (defined ($vm_view->summary->guest->toolsStatus)) {
            my $status = $vm_view->summary->guest->toolsStatus->val;
            print "VMware Tools:\t" . $toolsStatus{$status} . "\n";
        } else {
            print "VMware Tools:\tNot Known\n";
        }
        if (defined ($vm_view->summary->quickStats->overallCpuUsage)) {
            print "Cpu usage:\t" . $vm_view->summary->quickStats->overallCpuUsage . " MHz\n";
        } else {
            print "Cpu usage:\tNot Known\n";
        }
        if (defined ($vm_view->summary->quickStats->hostMemoryUsage)) {
            print "Host memory usage:\t" . $vm_view->summary->quickStats->hostMemoryUsage . " MB\n";
        } else {
            print "Host memory usage:\tNot Known\n";
        }
        if (defined ($vm_view->summary->quickStats->guestMemoryUsage)) {
            print "Guest memory usage:\t" . $vm_view->summary->quickStats->guestMemoryUsage . " MB\n";
        } else {
            print "Guest memory usage:\tNot Known\n";
        }
        if (defined ($vm_view->guest->net)) {
	    my @guest_net_array = $vm_view->guest->net;
	    my @guest_net_settings = @{$guest_net_array[0]};
	    my $output = '';
	    my $prefix = "IP Addresses:\t";
	    foreach my $guest_net_element (@guest_net_settings) {
		if (defined($guest_net_element->{'ipConfig'})) {
		    my $ipaddresses = $guest_net_element->{'ipConfig'}->{'ipAddress'};
		    foreach my $addr (@$ipaddresses) {
			next if (length($addr->{'ipAddress'}) > 16);
			$output = $output . $prefix . $addr->{'ipAddress'};
			$prefix = ",";
		    }
		}
	    }
	    print "$output\n";
	} else {
	    print "IP Addresses:\tNot Known\n";
	}

    }
}

#
# Determine currently-active vendor brand (lowercase).
#
sub VCD::getBrand()
{
    my $vendor = `grep VENDOR_LC_NAME /etc/COMPANY_BRAND_FILE`;
    $vendor =~ /export VENDOR_LC_NAME="(\w*)"/;
    return $1;
}

#
# Print information about vDisks in a VM
#
sub VCD::vdiskInfo($)
{
    my ($vmname) = @_;

    my $vmlist;
    die "No VM name specified" unless ($vmname);
    eval {
        $vmlist = Vim::find_entity_views(view_type => 'VirtualMachine',
                                         filter => { name => $vmname } ,
                                         properties => [ "name", "config", "summary", "runtime" ] );
    };
    die $@ if ($@);

    foreach (@$vmlist) {
        my $vm_view = $_;
    
        my $mor_host = $_->runtime->host;
        my $host = Vim::get_view(mo_ref => $mor_host);

        if (defined($vm_view->config) && defined ($vm_view->config->name)) {
            print "Name : " . $vm_view->config->name . "\n";
        }

        if (defined($vm_view->config) &&
            defined($vm_view->config->hardware) && 
            defined($vm_view->config->hardware->device)) {
            my $cnt = 0;
            my $devices = $vm_view->config->hardware->device;
            foreach my $dev (@$devices) {
                next unless $dev->isa("VirtualDisk");
                my $backing = $dev->backing;
                next unless ($backing->isa("VirtualDiskFlatVer2BackingInfo") ||
                             $backing->isa("VirtualDiskRawDiskMappingVer1BackingInfo") ||
                             $backing->isa("VirtualDiskPartitionedRawDiskVer2BackingInfo"));
                Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Name : "
                            . $dev->deviceInfo->label . "\n");
                if ($backing->isa("VirtualDiskRawDiskMappingVer1BackingInfo") ||
                    $backing->isa("VirtualDiskPartitionedRawDiskVer2BackingInfo")) {

                    if (defined($backing->compatibilityMode) && $backing->compatibilityMode eq "physicalMode") {
                        # pRDM disk
                        # check if this is cache device
                        my $lunUUID = $backing->lunUuid;
                        my $scsiLun = $host->config->storageDevice->scsiLun;
                        my $dtype = "pRDM";
                        my $canName = '';
                        my $lun;
                        foreach (@$scsiLun) {
                            if ($_->lunType eq "disk" && $_->uuid eq $lunUUID) {
                                $lun = $_;
                                if ($_->model eq "Flash Accel     ") {
                                    $dtype = "CACHE";
                                }
                                $canName = $_->canonicalName;
                            }
                        }

                        Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Type : "
                                    . $dtype . "\n");
                        Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Id : "
                                    . $canName . "\n");
                        if (defined($lun)) {
                            Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Blocks : "
                                        . $lun->capacity->block . "\n");
                              Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Blocksize : "
                                          . $lun->capacity->blockSize . "\n");
                          } else {
                              my $blocksize = 512;
                              my $blocks = ($dev->capacityInKB * 1024) / 512;
                              Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Blocks : "
                                          . $blocks . "\n");
                              Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Blocksize : "
                                          . $blocksize . "\n");
                          }
                    } else {
                        # vRDM disk
                        my $uuid = $backing->uuid;
                        $uuid =~ s/-//g;
                        Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Type : vRDM\n");
                        Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Id : naa."
                                    . $uuid . "\n");
                    }
                }
                if ($backing->isa("VirtualDiskFlatVer2BackingInfo")) {
                    my $uuid = $backing->uuid;
                    $uuid =~ s/-//g;
                    Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Type : VMDK\n");
                    Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Id : naa."
                                . $uuid . "\n");
                }
                Util::trace(0, $vm_view->name . " Disk[". $cnt ."]: Capacity : "
                            . ($dev->capacityInKB * 1024) . "\n");
                Util::trace(0, $vm_view->name ." Disk[". $cnt ."]: Path : "
                            ."Not Known\n");
                Util::trace(0, $vm_view->name ." Disk[".$cnt."]: Freespace : "
                            ."Not Known\n");
                $cnt++;
            }
        }
    }
}

#
# Obtain and print the vCenter version.
#
sub VCD::getvCenterVersion()
{
   my $vCenter_service_instance = Vim::get_service_instance(properties => ["content"]);
   my $vCenter_version = $vCenter_service_instance->content->about->version;
   print "$vCenter_version\n";
}

# 
# Annotations model:
# Annotations are of the following form
# Datacenter
#   (Host/VM based) Annotation field -- added once and available for all the Hosts/VMs
#   (Component based) Annotation value -- value written for each component.
#
#   i.e. After adding the field once, it will always be available. The decision whether or not the field
#   is valid or not is based on its value.
#   For Hosts -- Field is FA.HostAgent  --  Value is UUID of management console
#   For VMs   -- Field is FA.GosAgent   --  Value is UUID of management console
#

sub setAnnotation(@)
{
    my ($fieldKey, $fieldValue, $entity, $moType) = @_;
    my $serviceContent;
    my $customFieldsManager;
    my $customFields;

    eval {
        $serviceContent = Vim::get_service_content();
        $customFieldsManager = Vim::get_view(mo_ref => $serviceContent->customFieldsManager);
    };
    if ($@) {
        die "Error: Could not get customFieldsManager\n".$@;
    }

    $customFields = $customFieldsManager->field;

    my $keyIndex = '';
    foreach(@$customFields) {
        if($_->name eq $fieldKey) {
            my $k = $_->key;
            $keyIndex = $k;
        }
    }

    if ($keyIndex eq '') {
        # Custom field wasn't present previously. Add it first
        my $customFieldDef;
    
        eval {
            $customFieldDef = $customFieldsManager->AddCustomFieldDef(name => $fieldKey, moType => $moType);
        };
        if ($@) {
            die "Error: Could not add custom field\n".$@;
        }
        
        $keyIndex = $customFieldDef->key;
    }

    eval {
        $customFieldsManager->SetField(entity => $entity, key => $keyIndex, value => $fieldValue);
    };
    if ($@) {
        die "Error: Could not set component\n".$@;
    }
}

sub readAnnotation(@)
{
    my ($fieldKey, $fieldValue, $entity, $moType) = @_;
    my $serviceContent;
    my $customFieldsManager;
    my $customFields;

    if (!defined $fieldValue) {
        die "Error: Field value required\n";
    }

    eval {
        $serviceContent = Vim::get_service_content();
        $customFieldsManager = Vim::get_view(mo_ref => $serviceContent->customFieldsManager);
    };
    if ($@) {
        die "Error: Could not customFieldsManager\n".$@;
    }

    $customFields = $customFieldsManager->field;

    my $keyIndex = '';
    foreach(@$customFields) {
        if($_->name eq $fieldKey) {
            my $k = $_->key;
            $keyIndex = $k;
        }
    }
    if ($keyIndex eq '') {
        die "Error: No custom field named '$fieldKey' found.\n";
    }

    # In order to differentiate non existent field from changed field
    # If field is empty return      NOT_ANNOTATED - 2. 
    # If field is different return  ANNOTATED_BY_ANOTHER_CONSOLE - 0.
    # If field is the same return   ANNOTATED_BY_THIS_CONSOLE - 1.
    
    my ($value, $key);
    if (!defined $entity->customValue) {
        return NOT_ANNOTATED;
    }

    foreach ( @{$entity->customValue} ) {
        $key = $_->key;
        $value = $_->value;

        if ( $key eq $keyIndex ) {
            if ($value eq '') {
                return NOT_ANNOTATED;
            }

            if ($value ne $fieldValue) {
                return ANNOTATED_BY_ANOTHER_CONSOLE;
            } else {
                return ANNOTATED_BY_THIS_CONSOLE;
            }
        }
    }
    return NOT_ANNOTATED;
}

sub VCD::accessHostAnnotation
{
    my ($hostname, $annotationValue, $getOrSet)  = @_;
    my $host;

    eval {
        $host = Vim::find_entity_view(
                        view_type => 'HostSystem',
                        filter => { name => $hostname },
                    );
    };
    if ($@ || !defined $host) {
        die "Error: Could not get HostSystem entity\n".$@;
    }

    if ($getOrSet eq 'set') {
        setAnnotation('FA.HostAgent',$annotationValue,$host,'HostSystem');
    } else {
        my $annotationValid = readAnnotation('FA.HostAgent',$annotationValue,$host,'HostSystem');
        if ($annotationValid == ANNOTATED_BY_ANOTHER_CONSOLE) {
            die "Error: Host $hostname ownership changed\n";
        } elsif ($annotationValid == ANNOTATED_BY_THIS_CONSOLE ) {
            print "Status: Annotated\n";
        } else {
            print "Status: Not Annotated\n";
        }
    }

}

sub VCD::accessVMAnnotation
{
    my ($vmname, $annotationValue, $getOrSet) = @_;
    my $vm;

    eval {
        $vm = Vim::find_entity_view(
                        view_type => 'VirtualMachine',
                        filter => { "config.name" => $vmname },
                    );
    };
    if ($@ || !defined $vm) {
        die "Error: Could not get VirtualMachine entity\n".$@;
    }

    if ($getOrSet eq 'set') {
        setAnnotation('FA.GosAgent',$annotationValue,$vm,'VirtualMachine');
    } else {
        my $annotationValid = readAnnotation('FA.GosAgent',$annotationValue,$vm,'VirtualMachine');
        if ($annotationValid == ANNOTATED_BY_ANOTHER_CONSOLE) {
            die "Error: VM $vmname ownership changed\n";
        } elsif ($annotationValid == ANNOTATED_BY_THIS_CONSOLE ) {
            print "Status: Annotated\n";
        } else {
            print "Status: Not Annotated\n";
        }
    }
}

sub VCD::accessDataCenterAnnotation
{
    my ($dcname, $annotationValue,$getOrSet) = @_;
    my $datacenter;
    my $datacenter_views;
    
    eval {
        $datacenter_views = Vim::find_entity_views (view_type => 'Datacenter',
                                        filter => {name => $dcname});
    }; 
    if ($@ || !defined $datacenter_views) {
        die "Error: Could not get Datacenter entity\n".$@;
    }

    foreach(@$datacenter_views) {
        $datacenter = $_;
    }

    if ($getOrSet eq 'set') {
        setAnnotation('FA',$annotationValue,$datacenter,'Datacenter');
    } else {
        my $annotationValid = readAnnotation('FA',$annotationValue,$datacenter,'Datacenter');
        if ($annotationValid == ANNOTATED_BY_ANOTHER_CONSOLE) {
            die "Error: DataCenter $dcname ownership changed\n";
        } elsif ($annotationValid == ANNOTATED_BY_THIS_CONSOLE ) {
            print "Status: Annotated\n";
        } else {
            print "Status: Not Annotated\n";
        }
    }
}

sub VCD::getAnnotatedVMs(@)
{
    my ($dcname, $annotationValue) = @_;

    my $datacenter;

    if (defined $dcname) {
        my $datacenter_views = Vim::find_entity_views (view_type => 'Datacenter',
                                                       filter => {name => $dcname});
        die "Datacenter '$dcname' not found" unless (@$datacenter_views);

        foreach(@$datacenter_views) {
            $datacenter = $_;
        }
=pod
        # The following lines can be used when we want to manage FMC ownerships at datacenter level
        # This returns if the ownership of datacenter has changed.
        
        my $annotationValid = readAnnotation('FA',$annotationValue,$datacenter,'Datacenter');
        if ($annotationValid == ANNOTATED_BY_ANOTHER_CONSOLE) {
            die "Error: Datacenter $dcname ownership changed\n";
        }
=cut
        my $result = get_entities(view_type => 'VirtualMachine',
                                  obj => $datacenter);
        foreach (@$result) {
            my $obj_content = $_;
            my $mob = $obj_content->obj;
            my $obj = Vim::get_view(mo_ref=>$mob,
                                    properties => [ "name", "customValue" ] );

            my $annotationValid = readAnnotation('FA.GosAgent',$annotationValue,$obj,'VirtualMachine');
            if ($annotationValid == ANNOTATED_BY_THIS_CONSOLE) {
                print "VM : " . $obj->name. "\n";
            }
        }

    } else {
        die "Error: Datacenter name not defined\n";
    }
}

sub VCD::getAnnotatedHosts(@)
{
    my ($dcname, $annotationValue) = @_;

    my $datacenter;

    if (defined $dcname) {
        my $datacenter_views = Vim::find_entity_views (view_type => 'Datacenter',
                                                       filter => {name => $dcname});
        die "Datacenter '$dcname' not found" unless (@$datacenter_views);

        foreach(@$datacenter_views) {
            $datacenter = $_;
        }
=pod
        # The following lines can be used when we want to manage FMC ownerships at datacenter level
        # This returns if the ownership of datacenter has changed. 

        my $annotationValid = readAnnotation('FA',$annotationValue,$datacenter,'Datacenter');
        if ($annotationValid == ANNOTATED_BY_ANOTHER_CONSOLE) {
            die "Error: Datacenter ownership changed\n";
        }
=cut
        my $result = get_entities(view_type => 'HostSystem',
                                  obj => $datacenter);
        foreach (@$result) {
            my $obj_content = $_;
            my $mob = $obj_content->obj;
            my $obj = Vim::get_view(mo_ref=>$mob,
                                    properties => [ "name", "customValue" ] );
            my $annotationValid = readAnnotation('FA.HostAgent',$annotationValue,$obj,'HostSystem');
            if ($annotationValid == ANNOTATED_BY_THIS_CONSOLE) {
                print "Host : " . $obj->name . "\n";
            }
        }

    } else {
        die "Error: Datacenter name not defined\n";
    }

}

sub VCD::generateUUID
{
    my $uuidObject = new Data::UUID;
    my $uuid = $uuidObject->create_str();

    print "UUID : $uuid\n";
}

#
# Wait for a task to complete
# Args:
#   task - task we are waiting to complete
# Returns:
#   nothing
#
sub waitForTask {
    my ($task) = @_;
    my $state;
    my $lastState = "";

    while (1) {
        $state = $task->info->state->val;
        last if $state eq "success" || $state eq "error";
        select(undef, undef, undef, 0.25);      # sleep 0.25 secs
        $task->update_view_data();
    }

    if ( $task->info->state->val eq "error" ) {

          # This task has resulted in an error
          # Generate and throw a SOAP fault.
          my $faultMsgs = $task->info->error->fault->faultMessage;
          my $errMsgs = $task->info->error->localizedMessage . "\n";
          foreach my $fmsg (@$faultMsgs) {
              $errMsgs .= $fmsg->message . "\n";
          }

          my $soapFault = SoapFault->new;
          $soapFault->detail($task->info->error->fault);
          $soapFault->fault_string($errMsgs);
          die $soapFault;
    }
}


#
# Enable UUID for a VM
#
sub VCD::enableDiskUUID($)
{
    my ($vmname) = @_;

    my $vmlist;
    die "Error: No VM name specified" unless ($vmname);
    eval {
        $vmlist = Vim::find_entity_views(view_type => 'VirtualMachine',
                                         filter => { name => $vmname } ,
                                         properties => [ "config", "summary", "guest" ] );
    };
    die $@ if ($@);

    foreach (@$vmlist) {
        my $vm = $_;
	
	my $config = $vm->config;
	die "Error: No VM named '$vmname'" unless defined $config;
	my $extraConfig = $config->extraConfig;
	foreach my $optionValue (@$extraConfig) {
	    if ($optionValue->key eq "disk.EnableUUID") {
		if ($optionValue->value eq "true") {
		    return;
		} else {
		    last;
		}
	    }
	}

	# the attribute is not set
	my @mapping;
	push(@mapping, new OptionValue(key => "disk.EnableUUID", value => "true"));
	my $reconfig_spec = new VirtualMachineConfigSpec(extraConfig => \@mapping);
	eval {
	    my $task = $vm->ReconfigVM_Task(spec => $reconfig_spec);
	    waitForTask(Vim::get_view(mo_ref => $task));
	};
	if ($@) {
	    die "Error: Failed to update disk.EnableUUID for VM $vmname";
	}
    }
    print "Successfully updated disk.EnableUUID for VM $vmname\n";
}

1;
