#!/usr/bin/perl -w

#
# vCenter Discovery and Information Gathering
#
# Gather information from vCenter, Hosts and VMs.
# Supported operations are:
#   dclist - list of datacenters configured in the vcenter
#   clusterlist - list of clusters in the vcenter
#   hostlist - list of hosts in the vcenter
#   vmlist - list of VMs (grouped by hosts) in the vcenter
#   hostinfo - ESXi host information
#   maintmode - Check maintenance mode on a vSphere host
#   vminfo - VM information
#   vdiskinfo - VM disk info
#
use strict;
use warnings;
use Switch;

use VMware::VIRuntime;
use VMware::VILib;
use VMware::VICredStore;

use lib '/usr/lib/vmware-vcli/apps/general';
use VCD;

my %opts = (
	    'vmname' => {
		type => "=s",
		help => "VM name",
		required => 0,
	    },
	    'hostname' => {
		type => "=s",
		help => "Host name",
		required => 0,
	    },
	    'datacenter' => {
		type => "=s",
		help => "Name of the DataCenter",
		required => 0,
	    },
	    'cluster' => {
		type => "=s",
		help => "Name of the Cluster",
		required => 0,
	    },
	    'annotation' => {
		type => "=s",
		help => "Annotation of software to be written to a particular component",
		required => 0,
	    },
	    'operation' => {
		type => "=s",
		variable => "operation",
		help => "Operation to do: dclist, clusterlist, hostlist, vmlist, hostinfo, maintmode, vminfo, vdiskinfo, enableDiskUUID",
		required => 1,
	    },
	    'filter' => {
		type => "=s",
		variable => "filter",
		help => "VM Host List Filter options: cluster, datacenter",
		required => 0,
	    }
);

Opts::add_options(%opts);
Opts::parse();
Opts::validate();

Util::connect();

#
# ======= MAIN =============
#
my $operation = Opts::get_option('operation');
my $datacenter = Opts::get_option('datacenter');
my $cluster = Opts::get_option('cluster');
my $hostname = Opts::get_option('hostname');
my $vmname = Opts::get_option('vmname');
my $annotationValue = Opts::get_option('annotation');
my $filtertype = Opts::get_option('filter');

switch ($operation) {
    case "dclist" {
	VCD::listDC();
    }
    case "clusterlist" {
	VCD::listClusters();
    }
    case "hostlist" {
	VCD::listHosts($datacenter, $cluster);
    }
    case "vmfslist" {
	VCD::listVMFS($datacenter, $cluster);
    }
    case "vmhostlist" {
	VCD::vmHostList($vmname, $filtertype);
    }
    case "vmlist" {
	VCD::listHostsVMs($datacenter,  $cluster);
    }
    case "hostinfo" {
	VCD::hostInfo($hostname);
    }
    case "maintmode" {
	VCD::getMaintenanceMode($hostname);
    }
    case "vminfo" {
	VCD::vmInfo($vmname);
    }
    case "restartSfcbdWatchdog" {
        VCD::serviceSfcbdWatchdogRestart($hostname);
    }
    case "serviceGetSfcbdWatchdogStatus" {
        VCD::serviceGetSfcbdWatchdogStatus($hostname);
    }
    case "vdiskinfo" {
	VCD::vdiskInfo($vmname);
    }
    case "vcversion" {
        VCD::getvCenterVersion();
      }
    case "setHostAnnotation" {
        VCD::accessHostAnnotation($hostname, $annotationValue, "set");
    }
    case "setVMAnnotation" {
        VCD::accessVMAnnotation($vmname, $annotationValue, "set");
    }
    case "getHostAnnotation" {
        VCD::accessHostAnnotation($hostname, $annotationValue, "get");
    }
    case "getVMAnnotation" {
        VCD::accessVMAnnotation($vmname, $annotationValue, "get");
    }
    case "setDataCenterAnnotation" {
        VCD::accessDataCenterAnnotation($datacenter, $annotationValue, "set");
    }
    case "getDataCenterAnnotation" {
        VCD::accessDataCenterAnnotation($datacenter, $annotationValue, "get");
    }
    case "getAnnotatedHosts" {
        VCD::getAnnotatedHosts($datacenter, $annotationValue);
    }
    case "getAnnotatedVMs" {
        VCD::getAnnotatedVMs($datacenter, $annotationValue);
    }
    case "generateUUID" {
        VCD::generateUUID();
    }
    case "enableDiskUUID" {
	VCD::enableDiskUUID($vmname);
    }
    else {
	die "Invalid operation '$operation'";
    }
}

Util::disconnect();
