#!/usr/bin/perl -w

#
# vCenter Discovery and Information Gathering
#
# Gather information from vCenter, Hosts and VMs.
# Supported operations are:
#   dclist - list of datacenters configured in the vcenter
#   clusterlist - list of clusters in the vcenter
#   hostlist - list of hosts in the vcenter
#   vmlist - list of VMs (grouped by hosts) in the vcenter
#   hostinfo - ESXi host information
#   maintmode - Check maintenance mode on a vSphere host
#   vminfo - VM information
#   vdiskinfo - VM disk info
#
use strict;
use warnings;
use Switch;

use VMware::VIRuntime;
use VMware::VILib;
use VMware::VICredStore;

use lib '/usr/lib/vmware-vcli/apps/general';
use VCD;

Opts::parse();
Opts::validate();

Util::connect();

#
# Read next line of input and return it without the line terminator
#
sub read_stdin($)
{
   my ($operation) = @_;
   my $line;
   if (!($line = <STDIN>)) {
      die "broken pipe while $operation";
   }
   chop $line;
   return $line;
}

#
# ======= MAIN =============
#
my ($operation, $datacenter, $cluster, $hostname, $vmname, $done);

# Autoflush stdout
$| =1;

$done = 0;
while ((!$done) && ($operation = read_stdin("reading_operation"))) {
   switch ($operation) {
      case "no_op" {
      
      }
      case "exec" {
         my $cmd = read_stdin($operation);
         print `$cmd 2>/dev/null`;
      }
      case "dclist" {
         VCD::listDC();
      }
      case "clusterlist" {
         VCD::listClusters();
      }
      case "hostlist" {
         VCD::listHosts(undef, undef);
      }
      case "vmfslist" {
         VCD::listVMFS(undef, undef);
      }
      case "vmlist" {
         VCD::listHostsVMs(undef, undef);
      }
      case "hostlist_cluster" {
	  VCD::listHosts(undef, $cluster = read_stdin($operation));
      }
      case "vmfslist_cluster" {
        # Not used!
	  VCD::listVMFS(undef, $cluster = read_stdin($operation));
      }
      case "vmlist_cluster" {
         VCD::listHostsVMs(undef, $cluster = read_stdin($operation));
      }
      case "hostlist_dc" {
         VCD::listHosts($datacenter = read_stdin($operation), undef);
      }
      case "vmfslist_dc" {
        # Not used!
	  VCD::listVMFS($datacenter = read_stdin($operation), undef);
      }
      case "vmlist_dc" {
         VCD::listHostsVMs($datacenter = read_stdin($operation), undef);
      }
      case "hostinfo" {
         VCD::hostInfo($hostname = read_stdin($operation));
      }
      case "maintmode" {
         VCD::getMaintenanceMode($hostname = read_stdin($operation));
      }
      case "vminfo" {
         VCD::vmInfo($vmname = read_stdin($operation));
      }
      case "restartSfcbdWatchdog" {
         VCD::serviceSfcbdWatchdogRestart($hostname = read_stdin($operation));
      }
      case "serviceGetSfcbdWatchdogStatus" {
         VCD::serviceGetSfcbdWatchdogStatus($hostname = read_stdin($operation));
      }
      case "vdiskinfo" {
         VCD::vdiskInfo($vmname = read_stdin($operation));
      }
      case "enableDiskUUID" {
	VCD::enableDiskUUID($vmname = read_stdin($operation));
      }
      case "vcversion" {
         VCD::getvCenterVersion();
      }
      case "quit" {
         $done = 1;
      }
      else {
         die "Invalid operation '$operation'";
      }
   }
   print("\n\a\n");
}

Util::disconnect();
