<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * TODO: Many of the calls here are redundant with external_host.php,
 * tightly coupled with vSphere's vcenterdiscoverd.pl,
 * tightly coupled with the worker thread implementation, or all three.
 *
 * These functions aren't reusable because of the return format,
 * particularly the ones that start with "exec\n" or call
 * mercury_management_wa_exec (which adds the leading "exec\n").
 * Come back and revisit this during the Hyper-V implementation and try to split
 * the code up in a better way.
 *
 * Current best suggestion: Split the external_host.php functions into sub-
 * functions, one that returns the command and one that processes the result.
 * Then the worker thread cmd/cbk system can use the sub-functions and other
 * code can use the simple wrapper function.
 *
 * Or does it make more sense to fork PHP processes instead of the current way?
 *
 * Or if some external commands have to be handled in two places (not dozens),
 * that's not the end of the world.  That is the least-churn approach.
 *
 * @see mercury_management_wcmd_host_control_1 = properties
 * @see famc_external_api_host_get_properties
 * @see mercury_management_wa_exec
 * @see mercury_management_wa_host_software_vib_get
 */

/**
 *  Generate vcenterdiscoverd.pl command to obtain hostinfo for given host.
 */
function mercury_management_wcmd_hostinfo($host_name)
{
  return "hostinfo\n$host_name\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to obtain vminfo for given vm.
 */
function mercury_management_wcmd_vminfo($vm)
{
  return "vminfo\n$vm\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to obtain vdiskinfo for given vm.
 */
function mercury_management_wcmd_vdiskinfo($vm)
{
  return "vdiskinfo\n$vm\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to obtain host FA VIB status for given host.
 */
function mercury_management_wcmd_host_fa_vib_status($host_name, $host_username, $host_password)
{
  /**
   * Get the status of any and all FA vibs.
   * Using -n to specify the vib names will return an error if *any* are missing
   * and each esxcli command takes more than one second, so get all of the vibs
   * with a single command and filter down to what we actually want after that.
   * We specifically do not filter out non-FA vibs at this level because that
   * allows us to detect failures more reliably (no vibs = error).
   * @TODO: It's possible that some random vib could break this, now that we
   * return them all and since we have some format dependency.  See what we can
   * do to reduce the surface area.
   * The end result will look like this:
   *    tools-light|||5.0.0-0.0.469512
   *    netapp-flashaccel-cim|||1.2.0-20912270
   *    netapp-flashaccel|||1.2.0-20912270
   */
  $cmd  = "/usr/bin/esxcli -s '" . $host_name;
  $cmd .=      "' -u '" . $host_username;
  $cmd .=      "' -p '" . $host_password;
  $cmd .=      "' software vib get " . // all vibs
    "| grep -P 'Name: |Version: | failed:' " . // just these two fields or error message
    "| tr '\\n' ' ' " . // get rid of newlines to make the next part easier
    "| perl -p -e 's/Name:\s*(\S*)\s*Version:\s*(\S*)/$1|||$2\\n/g'";
  return "exec\n$cmd\n";
}


/**
 *  Parse mercury_management_wcmd_host_fa_vib_status (software vib get) output.
 */
function mercury_management_parse_vib_status(array $result) {
  $output = trim(implode("\n", $result));

  // We may have skipped the check because of annotations.
  // Assume installed for now, will validate via webservice call.
  // @see mercury_management_wa_cb_disc_host_hostinfo
  if ($output == 'SKIPPED') {
    return FAMC_HOST_BUNDLE_STATUS_INSTALLED;
  }

  // Detect this specific error message and display a different result.
  if (stripos($output, 'incorrect user name or password') !== false) {
    return "Error";
  }

  // If the result contains no vibs at all, there was some other error.
  // Most commonly "failed: Connection failure"
  if (empty($output) || strpos($output, '|||') === false) {
    return "CannotConnect";
  }

  // Check for the FA vibs.
  $brand = strtolower(mercury_utilities_get_fmc_brand_string());
  $main_installed = false;
  $cim_installed = false;
  if (stripos($output, "$brand-flashaccel|||") !== false) {
    $main_installed = true;
  }
  if (stripos($output, "$brand-flashaccel-cim|||") !== false) {
    $cim_installed = true;
  }
  if ($main_installed && $cim_installed) {
    return FAMC_HOST_BUNDLE_STATUS_INSTALLED;
  }

  // If exactly one vib is installed, we are in a special state.
  // But we specifically track and handle this state differently so that the
  // user can understand and self-correct this condition.
  if ($main_installed || $cim_installed) {
    return FAMC_HOST_BUNDLE_STATUS_PARTIAL_INSTALL;
  }

  // This error message is typical of missing vibs = uninstalled.
  // It only applies to vib get -n searches though, left here in case we go
  // back to using that.
  // Note that either/both vib missing from the output does not mean
  // uninstalled at this point, since we have to check for errors.
  /*if (stripos($output, "NoMatchError") !== false) {
    return FAMC_HOST_BUNDLE_STATUS_NOT_INSTALLED;
  }*/

  // No errors but also not found.
  return FAMC_HOST_BUNDLE_STATUS_NOT_INSTALLED;
}


/**
 * Parse mercury_management_wcmd_host_fa_vib_status (software vib get) output.
 * Return the version of the bundle installed on the system
 *
 * NOTE: The expectation is that this function is called only when 
 * the function mercury_management_parse_vib_status has returned INSTALLED.
 */
function mercury_management_parse_vib_version(array $result) {

  // Extract the VIB version from the output
  $vibVersion = '';
  if (!empty($result)) {
    foreach ($result as $value) {
      $value = trim($value);
      if (empty($value)) {
        continue;
      }

      // The line looks like this: netapp-flashaccel|||1.2.0-20912270
      $val_split = explode('|||', $value);
      if (count($val_split) > 1) {
        if (stripos($val_split[0], "flashaccel") !== false) {
          $vibVersion = trim($val_split[1]);
          break;
        }
      }
    }
  }

  return $vibVersion;
}

/**
 *  Generate vcenterdiscoverd.pl command to initate host adapter rescan.
 * @deprecated and unused.
 */
function mercury_management_wcmd_rescan_adapter($vc_hostname, $vc_username, $vc_password, $host_name, $adapter)
{
  $str  = "/usr/bin/esxcli"
        . " -s $vc_hostname"
        . " --username '" . $vc_username . "'"
        . " --password '" . $vc_password . "'"
        . " -h '"         . $host_name   . "'"
        . " storage core adapter rescan"
        . " --adapter '"  . $adapter     . "'"
        ;
  return "exec\n$str\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to obtain list of orphaned cache spaces.
 * @deprecated and unused.
 */
function mercury_management_wcmd_get_orphans($vc_hostname, $vc_username, $vc_password, $host_name)
{
  $str  = "/usr/bin/perl /usr/lib/vmware-vcli/apps/host/rdmDisk.pl"
        . " --url https://" . $vc_hostname . "/sdk/vimService"
        . " --username '" . $vc_username ."'"
        . " --password '" . $vc_password ."'"
        . " --hostname '" . $host_name   ."'"
        . " --operation getOrphanedCacheSpaces"
        ;
  return "exec\n$str\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to obtain host_control.py 1 data (Get Properties).
 */
function mercury_management_wcmd_host_control_1($host_name, $host_username, $host_password)
{
  $str  = "/usr/lib/vmware-vcli/apps/host/host_control.py"
        . " '" . $host_name     . "'"
        . " '" . $host_username . "'"
        . " '" . $host_password . "'"
        . " 1"
        ;
  return "exec\n$str\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to obtain host device list.
 */
function mercury_management_wcmd_esxcli_device_list($vc_hostname, $vc_username, $vc_password, $host_name)
{
  $str  = "/usr/bin/esxcli"
        . " -s $vc_hostname"
        . " --username '" . $vc_username . "'"
        . " --password '" . $vc_password . "'"
        . " -h '"         . $host_name   . "'"
        . " storage core device list"
        ;
  return "exec\n$str\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to obtain host path list.
 */
function mercury_management_wcmd_esxcli_path_list($vc_hostname, $vc_username, $vc_password, $host_name)
{
  $str  = "/usr/bin/esxcli"
        . " -s $vc_hostname"
        . " --username '" . $vc_username . "'"
        . " --password '" . $vc_password . "'"
        . " -h '"         . $host_name   . "'"
        . " storage core path list"
        ;
  return "exec\n$str\n";
}

/**
 *  Generate vcenterdiscoverd.pl command to obtain system coredump partition list.
 *  Example output:
 *  ~ # esxcli system coredump partition list
 *  Name                    Path                                        Active  Configured
 *  ----------------------  ------------------------------------------  ------  ----------
 *  mpx.vmhba32:C0:T0:L0:7  /vmfs/devices/disks/mpx.vmhba32:C0:T0:L0:7    true        true
 */
function mercury_management_wcmd_esxcli_system_coredump_partition_list($vc_hostname, $vc_username, $vc_password, $host_name)
{
  $str  = "/usr/bin/esxcli"
        . " -s $vc_hostname"
        . " --username '" . $vc_username . "'"
        . " --password '" . $vc_password . "'"
        . " -h '"         . $host_name   . "'"
        . " system coredump partition list"
        ;
  return "exec\n$str\n";
}
/**
 *  Generate vcenterdiscoverd.pl command to check VM credentials.
 */
function mercury_management_wcmd_vmrun_chk_cred($vc_hostname, $vc_username, $vc_password, $username, $password, $path)
{
  $str = "/usr/bin/vmrun -T esx"
       . " -h https://" . $vc_hostname . "/sdk"
       . " -u '" . $vc_username . "'"
       . " -p '" . $vc_password . "'"
       . " -gu '" . $username . "'"
       . " -gp '" . $password . "'"
       . " fileExistsInGuest '" . $path . "' \"C:\Windows\explorer.exe\"";
  return "exec\n$str\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to check if FA is installed on VM.
 */
function mercury_management_wcmd_vmrun_chk_for_fa($vc_hostname, $vc_username, $vc_password, $username, $password, $path)
{
  $brand = mercury_utilities_get_fmc_brand_string();
  $str = "/usr/bin/vmrun -T esx"
       . " -h https://" . $vc_hostname . "/sdk"
       . " -u '" . $vc_username . "'"
       . " -p '" . $vc_password . "'"
       . " -gu '" . $username . "'"
       . " -gp '" . $password . "'"
       . " fileExistsInGuest '" . $path . "'".' "C:\Program Files\\' . $brand . '\FlashAccel\FA-Service\FA-Service.dll"';

  // TODO: Resolve issues listed in burt782771 so we can use this.
  //. " runProgramInGuest '" . $path . "'".' "C:\\Windows\\System32\\wbem\\wmic.exe" "product where name=\"'. $brand . ' Flash Accel\" get name,version"';
  // @see mercury_management_parse_vmrun_for_fa_version
  return "exec\n$str\n";
}

/**
 *  Generate vcenterdiscoverd.pl command to submit a webservice command to a VM.
 * @see mercury_management_cache_operation
 * @see wsdl_cache_operation
 */
function mercury_management_wcmd_mm_wsdl($hostname, $username, $password, $method, $param = NULL, $port = NULL)
{
  $str = "/usr/bin/php /var/www/mercury/sites/all/modules/custom/mercury_management/mm_wsdl.php"
       . " --hostname '" . $hostname . "'"
       . " --username '" . $username . "'"
       . " --password '" . $password . "'"
       . " --method '"   . $method   . "'";
  if (isset($param)) {
    $str .= " --param '" . $param . "'";
  }
  if (isset($port)) {
    $str .= " --port '" . $port . "'";
  }
  return "exec\n$str\n";
}

function mercury_management_parse_mm_wsdl($result, $hostname, $suppress_error_responses = false) {

  // Prevent eval errors. Real empty results will be wrapped in quotes,
  // so completely empty is always an error.
  // This also represents a code bug, so ignore $suppress_error_responses.
  if (empty($result) || trim($result) == '') {
    mercury_utilities_fmc_log_message('VM Refresh', t('Empty service response from @vm.', array('@vm' => $hostname)), WATCHDOG_ERROR);
    return 'error_wsdl_call';
  }

  // The data is transferred via var_export, which means eval() to undo.
  // XXX: Eval is somewhat unsafe.  Using JSON would be better.
  eval("\$attrInfo = $result;");

  // If the result does not start with error_wsdl_call then it was a success.
  // Array results do not match the error pattern either, avoid WARNING.
  if (is_array($attrInfo) || strpos($attrInfo, 'error_wsdl_call') !== 0) {
    return $attrInfo;
  }

  // This was an error.  Pull out the message.
  // Currently the message comes after the pipe.
  $parts = explode('|', $attrInfo);
  unset($parts[0]);
  $message = implode('|', $parts);

  // Log the message.
  if (!empty($message) && !$suppress_error_responses) {
    mercury_utilities_fmc_log_message('Cache Mgmt', $message, WATCHDOG_ERROR);
  }

  // For backward compatibility, return only 'error_wsdl_call'.
  // This could use improvement in the future but for now this minimizes risk.
  return 'error_wsdl_call';
}

/**
 *  Generate vcenterdiscoverd.pl command to validate a FQDN.
 */
function mercury_management_wcmd_validate_fqdn($hostname)
{
  $str = "/usr/bin/host " . $hostname;
  return "exec\n$str\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to determine vCenter version.
 */
function mercury_management_wcmd_vcenter_version()
{
  return "vcversion\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to obtain list of datacenters
 * @deprecated and unused.
 */
function mercury_management_wcmd_dclist()
{
  return "dclist\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to obtain list of hosts in a datacenter
 * @deprecated and unused.
 */
function mercury_management_wcmd_hostlist_dc($dc)
{
  return "hostlist_dc\n$dc\n";
}


/**
 *  Generate vcenterdiscoverd.pl command to obtain list of hosts and VMs in a
 *  datacenter
 */
function mercury_management_wcmd_vmlist_dc($dc)
{
  return "vmlist_dc\n$dc\n";
}


/**
 *  Parse hostinfo output.
 */
function mercury_management_parse_hostinfo($result)
{
  $host_info_with_key = array();
  if (!empty($result)) {
    foreach ($result as $value) {
      $val_split = explode(':', $value);
      if (count($val_split) > 1) {
        $key = $val_split[0];
        $host_info_with_key[$key] = trim($val_split[1]);
      }
    }
  }

  return $host_info_with_key;
}


/**
 *  Parse vminfo output.
 */
function mercury_management_parse_vminfo($result)
{
  $vm_info_with_key = array();

  // The datastore name in vmPathName may contain a colon, which is bad because
  // we use a colon as a separator between key and value.
  // For now just re-merge anything that contains a colon in the values,
  // and eventually migrate to a better data format like JSON.
  // Note that mercury_management_parse_vdiskinfo etc. are also technically
  // vulnerable to the same problem but here and in mm_vmotion.php are the only
  // places where it impacts the operation of FAMC.
  // @see BURT 758398
  if (!empty($result)) {
    foreach ($result as $value) {
      $val_split = explode(':', $value);
      if (count($val_split) > 1) {
        $key = array_shift($val_split);
        $vm_info_with_key[$key] = trim(implode(':', $val_split));
      }
    }
  }

  return $vm_info_with_key;
}


/**
 *  Parse vdiskinfo output.
 */
function mercury_management_parse_vdiskinfo($vmname, $dataArr)
{
  $vdiskArr = array();

  if (!empty($dataArr)) {
    foreach ($dataArr as $value) {
      if (!stristr($value, "Disk")) {
        continue;
      }
      $val_split = explode(':', $value);
      if (trim($val_split[1]) == "Not Known") {
        return $vdiskArr;
      }

      // Sample output (note possible spaces and odd characters in VM name):
      // ab-_()+, =.cd Disk[0]: Name : Hard disk 1
      // ab-_()+, =.cd Disk[0]: Type : VMDK
      // ab-_()+, =.cd Disk[0]: Id : naa.6000C2b17f5442b5bcaf0f022c84bc4e
      // ab-_()+, =.cd Disk[0]: Capacity : 85899345920
      // ab-_()+, =.cd Disk[0]: Path : Not Known
      // ab-_()+, =.cd Disk[0]: Freespace : Not Known

      $n = sscanf($value, "$vmname Disk[%[0-9]]: %s : %[^\n]", $disk, $f_name, $f_val);

      if ($n != 3) {
        continue;
      }

      $f_val = trim($f_val);
      switch ($f_name) {
      case "Path":
        $vdiskArr[$disk]['Path'] = $f_val;
        break;
      case "Name":
        $vdiskArr[$disk]['name'] = $f_val;
        break;
      case "Type":
        $vdiskArr[$disk]['type'] = $f_val;
        break;
      case "Freespace":
        $vdiskArr[$disk]['freespace'] = $f_val;
        break;
      case "Capacity":
        $vdiskArr[$disk]['Capacity'] = $f_val;
        break;
      case "Blocks":
	$vdiskArr[$disk]['Blocks'] = $f_val;
	break;
      case "Blocksize":
	$vdiskArr[$disk]['Blocksize'] = $f_val;
	break;
      case "Id":
        $vdiskArr[$disk]['id'] = $f_val;
        break;
      }
    }
  }

  return $vdiskArr;
}


/**
 *  Parse output of rdmDisk.pl getOrphanedCacheSpaces.
 */
function mercury_management_parse_orphaned_cache_spaces($result)
{
  $orphanedCacheSpaces = array();

  foreach ($result as $output_line) {
    if (!empty($output_line) && stristr($output_line, "Error")) {
      return $output_line;
    }
    if (!empty($output_line) && stristr($output_line, "NAAID:")) {
      $orphanedCacheSpaceIDArray = explode(':',$output_line);
      if (count($orphanedCacheSpaceIDArray) > 1) {
        array_push($orphanedCacheSpaces, trim($orphanedCacheSpaceIDArray[1]));
      }
    }
  }

  return $orphanedCacheSpaces;
}


/**
 *  Parse output of host_control.py 1
 */
function mercury_management_parse_host_control_1($result)
{
  $host_info_with_key = array();

  foreach ($result as $value) {
    $val_split = explode(':', $value);
    if (count($val_split) > 1) {
      $key = $val_split[0];
      if ($key == 'HBTL') {
	$hbtl_split = explode('HBTL:', $value);
	$host_info_with_key[$key] = trim($hbtl_split[1]);
      } else {
	$host_info_with_key[$key] = trim($val_split[1]);
      }
    }
  }

  return $host_info_with_key;
}


/**
 *  Parse ESX host device and path lists.
 */
function mercury_management_parse_esx_device_and_path_lists($dataArr, $dataPathArr, $diagnosticPartitions)
{
  $deviceArr = array();
  if (empty($dataArr)) {
    return $deviceArr;
  }
  $validTransportDevices = array();
  $diagnosticDevices = array();

  /** Devices configured with a diagnostic partition shouldn't be displayed
   * Example output
   * ~ # esxcli system coredump partition list
   * Name                    Path                                        Active  Configured
   * ----------------------  ------------------------------------------  ------  ----------
   * mpx.vmhba32:C0:T0:L0:7  /vmfs/devices/disks/mpx.vmhba32:C0:T0:L0:7    true        true
   */
  foreach ($diagnosticPartitions as $outputLine) {
    if (!empty($outputLine) && stristr($outputLine, 'Error')) {
      break;
    }

    $deviceWithPartitionArr = explode(' ', $outputLine);
    $deviceWithPartition = $deviceWithPartitionArr[0];

    $diagnosticDeviceArr = explode(':', $deviceWithPartition);
    $diagnosticDevice = $diagnosticDeviceArr[0];
    $diagnosticDevices[$diagnosticDevice] = 1;
  }


  foreach ($dataPathArr as $outputLine) {
    $matches = array();

    # Looking for lines starting with 0 or more spaces, then Device:
    if (preg_match('/^ *Device: +(.+)$/', $outputLine, $matches)) {
      $device_name = trim($matches[1]);

    # Looking for lines starting with 0 or more spaces, then Transport:
    } else if (preg_match('/^ *Transport: +(.+)$/', $outputLine, $matches)) {
      $transport_type = trim($matches[1]);
      if (isset($device_name) &&
          ($device_name != '') &&
          ($device_name != 'No associated device') &&
          ($device_name != 'Runtime error') &&
          ($transport_type != 'usb') &&
          ($transport_type != 'ide') &&
          ($transport_type != 'block') &&
          ($transport_type != 'sata')) {
        $validTransportDevices[$device_name] = 1;
        unset($device_name);
      }
    }
  }

  $validDeviceFlag = 0;
  foreach ($dataArr as $value) {

    # Looking for non-empty lines with no leading spaces
    if (preg_match('/(^[^ ].*)$/', $value, $matches)) {
      $serial = trim($matches[1]);
      if ($validTransportDevices[$serial] == 1) {
        $deviceArr[$serial]['serial_no'] = $serial;
        $validDeviceFlag = 1;
# Stubbing this due to the decision to unfix burt 704093.  For now I want
# to make it really easy to add this back in.  If you delete this code,
# please also remove the related code above, the $diagnosticPartitions
# parameter, and fix the places that call here -- deriving that parameter
# adds an extra, now unnecessary step to host discovery/refresh.
#        if (!empty($diagnosticDevices[$serial])) {
#          if ($diagnosticDevices[$serial] == 1) {
#            unset($deviceArr[$serial]);
#            $validDeviceFlag = 0;
#          }
#        }
      } else {
        $validDeviceFlag = 0;
      }
    } else if ($validDeviceFlag == 1) {

      # Looking for lines that start with three spaces, then <Key>:, then
      # 0 or more spaces, then 0 or more non-space characters that we will
      # use as the value associated with <Key>.
      if (preg_match('/^   ([^:]+): *(.*)$/', $value, $matches)) {
        $key = $matches[1];
        $val = trim($matches[2]);
        $deviceArr[$serial][$key] = $val;
      } else {
        $validDeviceFlag = 0;
      }
    }
  }

  return $deviceArr;
}


/**
 *  Parse output of vmrun for checking VM credentials.
 */
function mercury_management_parse_vmrun_chk_cred($result)
{
  foreach ($result as $output) {
    if (!empty($output) && stristr($output, 'Error')) {
      if (stristr($output, 'password')) {
        return "Error";
      } else {
        return "CannotConnect";
      }
    }
  }

  return "Success";
}


/**
 *  Parse output of vmrun for checking for FA installed.
 */
function mercury_management_parse_vmrun_chk_for_fa($result)
{
  $driver_status = "Agent Not Installed";
  if (empty($result)) {
    return $driver_status;
  }

  foreach ($result as $output) {
    // @see @see mercury_management_wcmd_vmrun_chk_for_fa
    //if (!empty($output) && stristr($output, 'Flash Accel')) {
    if (!empty($output) && stristr($output, 'The file exists.')) {
      $driver_status = "Agent Installed";
    }
  }
  return $driver_status;
}

/**
 * Parse output of wmic query for the FA agent version
 * @see mercury_management_wcmd_vmrun_chk_for_fa
 */
function mercury_management_parse_vmrun_for_fa_version($result)
{
  throw new \Exception('Not available, see bug 782771');
  $driver_version = "Not Known";
  if (empty($result)) {
    return $driver_version;
  }

  foreach ($result as $output) {
    if (!empty($output) && stristr($output, 'Flash Accel')) {
      $parts = explode(",", $output);
      $driver_version = $parts[2];
    }
  }
  return $driver_version;
}


/**
 *  Parse output of host for validating FQDN.
 */
function mercury_management_parse_wcmd_validate_fqdn($result, &$ip)
{
  if (empty($result)) {
    return false;
  }

  foreach ($result as $output) {
    if (stristr($output, 'has address')) {
      $parts = explode(' ', $output);
      $ip = $parts[3];
      return true;
    }
  }

  return false;
}


/**
 *  Parse output of vcd command vmlist_dc
 */
function mercury_management_parse_wcmd_vmlist_dc($result)
{
   $rv = array();

   $count = 0;
   foreach ($result as $output_line) {
      $exploded = explode(':', $output_line);
      $type = strtolower(trim(array_shift($exploded)));
      $name = trim(implode(':', $exploded));
      if ($type == 'host' || $type == 'vm') {
         $rv[$count]['type'] = $type;
         $rv[$count]['name'] = $name;
         $count++;
      }
   }

   return $rv;
}
