<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * Functions to help with handling generic nodes consistently.
 */

/**
 * Returns an array of all parent nodes for the node and the node itself
 * @param stdClass $node
 * @return array of nodes, keyed with vm host datacenter vcenter.
 */
function mercury_management_get_ancestors(stdClass $node, &$info = array()) {
  $infix = mercury_management_get_function_infix($node);
  $function = "mercury_management_get_{$infix}_ancestors";
  // The VM version will call the host version, etc.
  return $function($node);
}

/**
 * Returns an array of decrypted credentials for a node.
 * @param stdClass $node
 * @param boolean $include_ancestors if true, also returns ancestor credentials
 * @return array of credentials
 */
function mercury_management_get_credentials(stdClass $node, $include_ancestors = false) {

  $infix = mercury_management_get_function_infix($node);
  $credentials_function = "mercury_management_get_{$infix}_credentials";

  // Not all layers have credentials, but still return the node for that layer.
  // It helps with chaining and often we need the ancestor node objects, too.
  if (is_callable($credentials_function)) {
    $result = $credentials_function($node);
  } else {
    $result = array();
  }
  $result['node'] = $node;

  if (!$include_ancestors) {
    return $result;
  }

  $ancestors = mercury_management_get_ancestors($node);
  foreach($ancestors as $key => $ancestor_node) {
    // Skip VM "ancestor" if already in $result, etc.
    if ($key == $infix) {
      continue;
    }
    $result[$key] = mercury_management_get_credentials($ancestor_node);
  }
  return $result;
}

/**
 * Convert a discovery state integer into user-oriented text.
 * @param stdClass $node Node to check
 * @return string descriptive text
 */
function mercury_management_get_discovery_state_text(stdClass $node) {
  $field_name = 'field_' . mercury_management_get_function_infix($node) . '_discovery_state';

  // '0' is a valid value so don't use empty().
  $field = NULL;
  if (isset($node->$field_name)) {
    $field = $node->$field_name;
  }
  if (!isset($field['und'][0]['value'])) {
    return t('Error: Invalid discovery state.');
  }

  switch ($field['und'][0]['value']) {
    case FAMC_DISCOVERY_STATE_NAME_ONLY:
      // This should never happen.
      return t('Only the name is available.');
    case FAMC_DISCOVERY_STATE_VCENTER:
      return t('Partial discovery with errors. Please check the logs for details.');
    case FAMC_DISCOVERY_STATE_REACHABLE:
      // If installed this is an error case, so don't sound too happy.
      // In that case this text will be followed by error details.
      return t('Powered on, only vSphere information available.');
    case FAMC_DISCOVERY_STATE_API_READY:
      return t('Flash Accel service available and returning data.');
  }
  return t('Unknown');
}

/**
 * Checks to see if we can use the SOAP/CIM API for a given object.
 * @param stdClass $node
 * @return string empty on success or an error message
 */
function mercury_management_has_fa_api_error(stdClass $node) {
  $field_name = 'field_' . mercury_management_get_function_infix($node) . '_discovery_state';
  $field = $node->$field_name;
  if ($field['und'][0]['value'] >= FAMC_DISCOVERY_STATE_API_READY) {
    return ''; // no error
  }

  // This condition can either be caused by the agent not being installed or
  // a communication error during discovery.
  // If there is a discovery error, use that in the error message.
  $type = mercury_management_get_printable_node_type($node);
  $details_name = 'field_' . mercury_management_get_function_infix($node) . '_discovery_error';
  $details_field = $node->$details_name;
  $error = '';
  if (!empty($details_field['und'][0]['value'])) {
    $details = (!empty($details_field['und'][0]['value'])) ?  : 'Unknown';
    $error = t('@type @title has a Flash Accel API communication problem: @details', array(
      '@type' => $type, '@title' => $node->title, '@details' => $details_field['und'][0]['value'])
    );
  } else {
    // Otherwise the API is not available because the agent is not installed.
    // Report on agent status to aid in debugging and message clarity.
    $agent_name = 'field_driver_status';
    if ($node->type == 'host') {
      $agent_name = 'field_bundle_status';
    }
    $agent_field = $node->$agent_name;
    $error = t('@type @title action failed because of agent install status: @details. Please install the agent before continuing.', array(
      '@type' => $type, '@title' => $node->title, '@details' => $agent_field['und'][0]['value'])
    );
  }

  // Log and return the message.
  mercury_utilities_fmc_log_message("$type FA API Error", $error, WATCHDOG_ERROR);
  return $error;
}

/**
 * Checks to see if we can use the vSphere APIs (esxcli, vmrun).
 * Install state in independent of vSphere state.
 * @param stdClass $node
 * @return string empty on success or an error message
 */
function mercury_management_has_vsphere_api_error(stdClass $node) {
  $field_name = 'field_' . mercury_management_get_function_infix($node) . '_discovery_state';
  $field = $node->$field_name;
  if ($field['und'][0]['value'] >= FAMC_DISCOVERY_STATE_REACHABLE) {
    return ''; // no error
  }

  // Pull in the discovery details for a better error message and log entry.
  $details_name = 'field_' . mercury_management_get_function_infix($node) . '_discovery_error';
  $details_field = $node->$details_name;
  $details = (!empty($details_field['und'][0]['value'])) ? $details_field['und'][0]['value'] : 'Unknown';
  $type = mercury_management_get_printable_node_type($node);

  // Log and return the message.
  $error = t('@type @title has a communication problem: @details', array('@type' => $type, '@title' => $node->title, '@details' => $details));
  mercury_utilities_fmc_log_message("$type vSphere API Error", $error, WATCHDOG_ERROR);
  return $error;
}

/**
 * Returns the function name infix for a given type of node.
 * @param stdClass $node
 * @return string
 * @throws Exception on unknown type
 */
function mercury_management_get_function_infix(stdClass $node) {
  switch($node->type) {
    case 'data_center': return 'datacenter';
    case 'host': return 'host';
    case 'vcenter': return 'vcenter';
    case 'virtual_machine': return 'vm';
  }
  throw new Exception("{$node->type} not supported!");
}

/**
 * Return the type of the node in a format suitable for display to a user.
 * @param stdClass $node
 * @return string
 * @throws Exception on unknown type
 */
function mercury_management_get_printable_node_type(stdClass $node) {
  switch($node->type) {
    case 'data_center': return 'Datacenter';
    case 'host': return 'Host';
    case 'vcenter': return 'vCenter';
    case 'virtual_machine': return 'VM';
  }
  throw new Exception("{$node->type} not supported!");
}

/**
 * Tries to connect to a given resource to validate credentials.
 * @param stdClass $node
 * @param boolean $validate_ancestors If true it will also validate ancestors.
 * @param string $address Optional, will use node data if empty.
 * @param string $username Optional, will use node data if empty.
 * @param string $password Optional, will use node data if empty.
 * @param string $port Optional, will use node data if empty.
 * @return mixed Boolean true or an error string.
 */
function mercury_management_validate_credentials(stdClass $node, $validate_ancestors = true, $address = NULL, $username = NULL, $password = NULL, $port = NULL) {

  // The same vCenter / host is likely to be validated multiple times
  // in the same request.  Avoid duplicate work.
  static $cache = array();

  // This is safe for now, but be careful in the future.
  // Almost all validations will be based on the node in the database,
  // and the very few that are not are still consistent per-request.
  // Do not need to factor in ancestors here, they will cache individually.
  $cacheKey = $node->nid;

  if (isset($cache[$cacheKey])) { // cached
    $result = $cache[$cacheKey];
  } else { // not cached
    // User/pass are only passed in during pre-save form validation.
    if (empty($username) && empty($password)) {
      $credentials = mercury_management_get_credentials($node);
      $address = $credentials['address'];
      $username = $credentials['username'];
      $password = $credentials['password'];
      if (isset($credentials['port'])) {
        $port = $credentials['port'];
      }
    }

    switch ($node->type) {
      case 'virtual_machine':
        $result = famc_external_api_vm_validate_credentials($node, $address, $username, $password, $port);
        break;

      case 'host':
        $result = famc_external_api_host_validate_credentials($node, $address, $username, $password);
        break;

      case 'vcenter':
        $result = famc_external_api_manager_validate_credentials($node, $address, $username, $password);
        break;

    }

    // Save new result to cache.
    $cache[$cacheKey] = $result;
  }

  // Also validate ancestors when $validate_ancestors, but only if it matters.
  // Need to execute this block regardless of the cache state.
  $ancestor_type = NULL;
  if ($node->type == 'virtual_machine') {
    $ancestor_type = 'host';
  } elseif ($node->type == 'host') {
    $ancestor_type = 'vcenter'; // At least for now, nothing in between.
  }
  if ($result === true && $validate_ancestors && $ancestor_type) {
    $ancestors = mercury_management_get_ancestors($node);
    return mercury_management_validate_credentials($ancestors[$ancestor_type]);
  }

  return $result;
}
