<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

//Define the path for the Export/Import files
define('EXPORT_IMPORT_PATH', '/var/www/mercury/sites/default/files/export_import/');
define('EXPORT_IMPORT_BROWSER_PATH', '/sites/default/files/export_import/');

/**
 * Contains helper functions to aid in the import/export process.
 */
class MercuryImportExport
{

  // Map between sequential CSV columns and associative field names.
  // Note that any new columns have to go at the end or it will break importing
  // from older versions!
  public static $CSV_field_map = array(
    'type',
    'title',
    'username',
    'password',
    'install_port',
    'address_custom',
  );

  /**
   * Creates the table needed to store credentials for nodes not yet discovered.
   * @TODO Get this table into the .sql file after we retake ownership.
   * Or even better, start using .install files properly...
   */
  protected static function create_temporary_credential_table() {
    static $once;
    if ($once) {
      return;
    }
    $once = true;

    $exists = db_query("show tables like 'mercury_utilities_import_saved_credentials'")->fetchField();
    if (!$exists) {
      drupal_install_schema('mercury_utilities');
    }
  }

  /**
   * Helper function for MercuryImportExport::export_gather().
   * We only export a given node if it is not using the default settings.
   * @param stdClass $node The Node to check
   * @param string $default_user The global default user
   * @param string $default_pass The global default password
   * @param type $default_port The global default port
   * @return array of non-default attributes, empty array if default.
   */
  protected static function export_check_default_auth($node, $default_user, $default_pass, $default_port = FAMC_WEBSERVICE_DEFAULT_PORT) {
    $result = array();

    // If there is a custom IP address, save it.
    $address_custom = mercury_utilities_get_node_field($node, 'vm_address_custom');
    if (!empty($address_custom)) {
      $result['address_custom'] = $address_custom;
    }

    // If there is a port and it's non-default, save it.
    $port = mercury_utilities_get_node_field($node, 'install_port');
    if (!empty($port) && $port != $default_port) {
      $result['install_port'] = $port;
    }

    /**
     * BOTH $user and $pass must be set
     * AND at least one of them must not be the default.
     * Same user but different password etc. = must export,
     * and it shouldn't be possible to enter only one field.
     */
    $user = mercury_utilities_get_node_field($node, 'username');
    $pass = mercury_utilities_get_node_field($node, 'password');
    if (
      !empty($user)
      && !empty($pass)
      && ($user != $default_user || $pass != $default_pass)
    ) {
      $result['username'] = $user;
      $result['password'] = $pass;
    }

    return $result;
  }

  /**
   * Helper function for mercury_utilites_generate_export_file().
   * Collects export data from various sources and standardizes it.
   * @return type
   */
  public static function export_gather() {
    $data = array();

    // The first line of the CSV is the Version number.
    // It's used to determine CSV format on import.
    $data[] = array('version', mercury_utilities_get_fmc_version_string());

    // Store a known string, encrypted using the key.
    // This allows us to detect key differences which would corrupt data.
    $data[] = self::export_gather_encryption_check();

    /*
     * IMPORTANT: Do not IMPORT the vCenter info for security reasons.
     * We intentionally require that the vCenter info be entered normally before
     * any import can take place.
     * However, we must EXPORT the information anyway for a validation step.
     */
    $data[] = self::export_gather_vCenter();

    // Start with the global default Host and VM settings.
    // These will be used in comparisons later, too.
    $host_username = variable_get('host_username', '');
    $host_password = variable_get('host_password', '');
    $vm_username = variable_get('vm_username', '');
    $vm_password = variable_get('vm_password', '');
    $data[] = array(
      'type' => 'default_host',
      'username' => $host_username,
      'password' => $host_password,
    );
    $data[] = array(
      'type' => 'default_virtual_machine',
      'username' => $vm_username,
      'password' => $vm_password,
    );

    // And finally any Host- or VM-specific overrides.
    $hosts = mercury_utilities_node_load_all('host');
    if (!$hosts) {
      return $data; // Implies no VMs.
    }
    foreach ($hosts as $node) {
      $tempRow = self::export_check_default_auth($node, $host_username, $host_password);
      if (empty($tempRow)) {
        continue; // Nothing to export!
      }
      $data[] = array_merge(array(
        'type' => 'host',
        'title' => $node->title,
        ), $tempRow);
    }
    unset($hosts);

    $vms = mercury_utilities_node_load_all('virtual_machine');
    if (!$vms) {
      return $data;
    }
    foreach ($vms as $node) {
      $tempRow = self::export_check_default_auth($node, $vm_username, $vm_password);
      if (empty($tempRow)) {
        continue; // Nothing to export!
      }
      $data[] = array_merge(array(
        'type' => 'virtual_machine',
        'title' => $node->title,
        ), $tempRow);
    }
    unset($vms);

    return $data;
  }

  /**
   * Encrypt a known string and return it wrapped in an export array.
   * This allows us to confirm that the export and import used the same key.
   * @return array for export
   */
  protected static function export_gather_encryption_check() {
    module_load_include('inc', 'mercury_utilities', 'includes/fmc_cipher');
    return array(
      'encryption_check',
      fmc_cipher::encrypt('This is a known string.'),
    );
  }

  /**
   * Fetch the vCenter data as an array
   * @return array of vCenter data
   */
  protected static function export_gather_vCenter() {
    // An invalid vCenter redirects away to the vCenter settings page in init(),
    // so we don't need any redundant checks here.
    $vCenter = mercury_management_vcenter_details();

    return array(
      'type' => 'vCenter',
      'title' => mercury_utilities_get_node_field($vCenter, 'hostname'),
      'username' => mercury_utilities_get_node_field($vCenter, 'username'),
      'password' => mercury_utilities_get_node_field($vCenter, 'password'),
    );
  }

  /**
   * The gathered export data is associative but CSVs are not!
   * Convert the associative data to consistent numbered columns.
   * @param array $row One row of associative data for export.
   * @return array with all data in the correct sequential columns.
   */
  public static function export_normalize($row) {
    // The version string and potentially others are already sequential = done.
    if (isset($row[0])) {
      return $row;
    }

    $newRow = array();
    foreach (self::$CSV_field_map as $field) {
      // If the field in this row is missing,
      // we must still insert an empty string.
      if (isset($row[$field])) {
        $newRow[] = (string) $row[$field];
      } else {
        $newRow[] = '';
      }
    }

    return $newRow;
  }

  /**
   * Helper function to actually apply the changes to the node and update db,
   * @param array $assoc import row
   * @param stdClass $node The Node to check
   * @return int count of items modified
   */
  protected static function import_apply_data_to_node($assoc, $node) {
    // Track whether we updated this node.
    $updated = false;

    // Only import if the current ones are empty.
    $user = mercury_utilities_get_node_field($node, 'username');
    $pass = mercury_utilities_get_node_field($node, 'password');
    if (!$user && !$pass) {
      if (!empty($assoc['username']) && !empty($assoc['password'])) {
        mercury_utilities_set_node_field($node, 'username', $assoc['username']);
        mercury_utilities_set_node_field($node, 'password', $assoc['password']);
        $updated = true;
      }
    }

    // If this is a VM, also try to set a port and custom IP address.
    if ($assoc['type'] == 'virtual_machine') {
      $address_custom = mercury_utilities_get_node_field($node, 'vm_address_custom');
      if (empty($address_custom)) {
        if (!empty($assoc['address_custom'])) {
          mercury_utilities_set_node_field($node, 'vm_address_custom', $assoc['address_custom']);
          $updated = true;
        }
      }
      $port = mercury_utilities_get_node_field($node, 'install_port');
      if (empty($port) || $port == FAMC_WEBSERVICE_DEFAULT_PORT) {
        if (!empty($assoc['install_port']) && intval($assoc['install_port']) > 0) {
          mercury_utilities_set_node_field($node, 'install_port', intval($assoc['install_port']));
          $updated = true;
        }
      }
    }

    /*
     * Save the updated node if anything changed.
     *
     * Note that in the mercury_utilities_node_presave code path this is
     * technically redundant, but for some reason it doesn't work without the
     * duplicate node_save call! There may be interference from some other code.
     */
    if ($updated) {
      node_save($node);
    }

    return $updated;
  }

  /**
   * Loop through all credentials in temporary storage and try to apply them.
   */
  public static function import_apply_all_saved_credentials() {
    self::create_temporary_credential_table();
    $stored = db_query("SELECT * FROM {mercury_utilities_import_saved_credentials}")->fetchAllAssoc('id', PDO::FETCH_ASSOC);

    foreach($stored as $assoc) {
      // Try to find a node that matches the title.
      $query = new EntityFieldQuery();
      $query->entityCondition('entity_type', 'node')
        ->entityCondition('bundle', $assoc['type'])
        ->propertyCondition('title', $assoc['title'])
        ->propertyCondition('status', 1);
      $result = $query->execute();

      // Not discovered yet, leave it be.
      if (empty($result['node'])) {
        mercury_utilities_discovery_progress_register_ticks(1);
        continue;
      }

      // Load the data for the found node.
      $result_inner = reset($result['node']);
      $node = node_load($result_inner->nid);

      // Process the node.
      self::import_apply_data_to_node($assoc, $node);

      // Delete the stored data to avoid any accidents.
      // User must run import again if they blow out settings.
      db_delete('mercury_utilities_import_saved_credentials')
        ->condition('type', $node->type)
        ->condition('title', $node->title)
        ->execute();

      // Update progress bar.
      mercury_utilities_discovery_progress_register_ticks(1);
    }
  }

  /**
   * Approximate the amount of work to do for discovery.
   */
  public static function import_apply_all_saved_credentials_get_ticks() {
    self::create_temporary_credential_table();

    // One point per saved credential to process.
    return db_select('mercury_utilities_import_saved_credentials')
      ->countQuery()->execute()->fetchField();
  }

  /**
   * Take a row of CSV data and apply it to the database.
   * Note that this will not overwrite existing data!
   * @param array $row one CSV line
   * @return int count of items modified
   */
  public static function import_row($row) {
    // The first line is the headers.  Ignore.
    if ($row[0] == 'type') {
      return 0; // No modifications.
    }

    // The second line is the version string.
    // $version is reserved for future use with format changes.
    static $version;
    if (empty($version)) {
      if ($row[0] == 'version') {
        $version = self::import_validate_version($row);
        return 0; // No modifications.
      }

      // Not a version string.  Most likely the import is missing the row.
      throw new Exception(t('Import file is missing version string.'));
    }

    // The third line is the encryption key check.
    // Make sure the encryption key matches the export to avoid data corruption.
    static $encryptionKeyValid;
    if (empty($encryptionKeyValid)) {
      if ($row[0] == 'encryption_check') {
        $encryptionKeyValid = self::import_validate_encryption_check($row);
        return 0; // No modifications.
      }

      // Not an encryption_check.  Most likely the import is missing the row.
      throw new Exception(t('Import file is missing encryption key check.'));
    }

    // All remaining lines contain object or variable data.

    // array_combine will fail if $row is missing new columns.
    $row = array_pad($row, count(self::$CSV_field_map), '');

    // $row is sequential.  Convert to associative.
    $assoc = array_combine(self::$CSV_field_map, $row);

    // Before allowing any data changes, we must validate the vCenter
    // credentials against what is currently in the node.
    static $vCenterValid;
    if (empty($vCenterValid)) {
      if ($assoc['type'] == 'vCenter') {
        $vCenterValid = self::import_validate_vCenter($assoc);
        return 0; // No modifications.
      }

      // Not a vCenter.  Most likely the import is missing the row.
      throw new Exception(t('Import file is missing vCenter details.'));
    }

    // Delegate the remaining work based on storage type.
    switch($assoc['type']) {
      case 'default_host':
      case 'default_virtual_machine':
        return self::import_row_variable($assoc);
      case 'host':
      case 'virtual_machine':
        return self::import_row_node($assoc);
      default:
        throw new Exception('Unhandled import type "' . check_plain($assoc['type']) . '"');
    }
  }

  /**
   * Save auth info to a Drupal node.
   * @param array $assoc import row
   * @return int count of items modified
   */
  protected static function import_row_node($assoc) {
    // Make sure it's even possible to continue, might be bad data.
    if (empty($assoc['type']) || empty($assoc['title'])) {
      return 0;
    }

    // Try to find a node that matches the title.
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')
      ->entityCondition('bundle', $assoc['type'])
      ->propertyCondition('title', $assoc['title'])
      ->propertyCondition('status', 1);
    $result = $query->execute();

    // The node has not been discovered yet.
    // Store the credentials in a temporary location and import on discovery.
    if (empty($result['node'])) {
      return self::import_row_store_temporary_data($assoc);
    }

    // Load the data for the found node.
    $result_inner = reset($result['node']);
    $node = node_load($result_inner->nid);

    // Applies the changes and calls node_save.
    return (int) self::import_apply_data_to_node($assoc, $node);
  }

  /**
   * Store the node's credentials in a temporary location until the node is
   * created. This covers import before discovery.
   * @param array $assoc import row
   * @return int count of items modified
   */
  protected static function import_row_store_temporary_data($assoc) {
    self::create_temporary_credential_table();

    // Merge the value aka upsert aka IODKU.
    $changed = db_merge('mercury_utilities_import_saved_credentials')
      ->key(array(
        'type' => $assoc['type'],
        'title' => $assoc['title']
      ))
      ->fields(array(
        'username' => $assoc['username'],
        'password' => $assoc['password'],
        'address_custom' => $assoc['address_custom'],
        'install_port' => $assoc['install_port'],
      ))
      ->execute();

    // $changed is 1 on insert, 2 on update regardless of whether it changed.
    // For now the best we can do is count only inserts as modifications.
    return (int) ($changed === 1);
  }

  /**
   * Save auth info to a Drupal variable.
   * @param array $assoc import row
   * @return int count of items modified
   */
  protected static function import_row_variable($assoc) {
    // On the off chance that the export contains an invalid row, skip.
    if (empty($assoc['username']) || empty($assoc['password'])) {
      return 0;
    }

    $prefix = 'host_';
    if ($assoc['type'] == 'default_virtual_machine') {
      $prefix = 'vm_';
    }

    // Only import if the current ones are empty.
    if (
      variable_get($prefix . 'username', FALSE)
      || variable_get($prefix . 'username', FALSE)
    ) {
      return 0;
    }

    // Assign the variables.
    variable_set($prefix . 'username', $assoc['username']);
    variable_set($prefix . 'password', $assoc['password']);

    return 1;
  }

  /**
   * Makes sure that the import and export FMC deployments use the same key.
   * @param array $importValue from CSV
   * @return boolean true if they match
   * @throws Exception
   */
  protected static function import_validate_encryption_check($importValue) {
    $currentValue = self::export_gather_encryption_check();

    // Ignores anything that is in the import but no longer gets exported.
    $diff = array_diff_assoc($currentValue, $importValue);
    if (empty($diff)) {
      return true;
    }

    throw new Exception(t('Encryption check did not match. The FMC that exported this file may have been using a different encryption key.'));
  }

  /**
   * Makes sure that the import and export FMC deployments use the same vCenter.
   * @param array $importValue from CSV
   * @return boolean true if they match
   * @throws Exception
   */
  protected static function import_validate_vCenter($importValue) {
    $currentValue = self::export_gather_vCenter();

    // Ignore the username because it's case insensitive and that can cause
    // some frustrating error messages.
    unset($importValue['username'],$currentValue['username']);

    // The CSV data must exactly match the current vCenter.
    // Ignores anything that is in the import but no longer gets exported.
    $diff = array_diff_assoc($currentValue, $importValue);
    if (empty($diff)) {
      return true;
    }

    throw new Exception(t('The vCenter credentials did not match. The FMC that exported this file may have been associated with a different vCenter.'));
  }

  /**
   * Makes sure that the import and export CSV format versions are compatible.
   * @param array $importValue from CSV
   * @return boolean true if they match
   * @throws Exception
   */
  protected static function import_validate_version($importValue) {
    // Only care about major and minor.
    // Build number etc. doesn't matter for import.
    $currentValue = floatval(mercury_utilities_get_fmc_version_string());
    $importValue = floatval($importValue[1]);

    // First valid export version is 1.2, but account for floating point math.
    if ($importValue < 1.15) {
      throw new Exception(t('Invalid version string detected.'));
    }

    // Imports are backward compatible but not future-compatible.
    // Ex: Copper exports should not be imported into Barium.
    // Again watch out for floating point math.
    if ($importValue > ($currentValue + 0.05)) {
      throw new Exception(t('The import file is not compatible with the current version of FMC. Please import into an updated FMC instead.'));
    }

    return $importValue;
  }

}

function mercury_utilities_export_import_form() {
    $form = array();

    $latest_export_file = mercury_utilities_get_latestFile(EXPORT_IMPORT_PATH);

    $form['export_import_title'] = array(
        '#type' => 'markup',
        '#markup' => t('<h2>Export / Import</h2>'),
    );
    $form['export_import_description'] = array(
        '#type' => 'markup',
        '#markup' => t('<b>Use this page to transfer FMC configuration data, such as user names and passwords, from one FMC instance to another.  Click Generate Export File to export configuration data from the old FMC into a file. After deploying the new FMC, click Import Configuration to import the data into the new FMC.</b><br/><br/>'),
    );
    $form['export'] = array(
        '#type' => 'fieldset',
        '#title' => 'Export',
        '#collapsible' => false,
        '#collapsed' => false,
        '#suffix' => '<br/>',
    );

    if ( $latest_export_file and $latest_export_file != 'not_found' ) {
        $form['export']['export_text'] = array(
            '#type' => 'markup',
            '#markup' => t('<div>An <b>FAMC-Export-Data</b> file is ready for downloading. Click the <b>Export Configuration</b> button to save this file to your desktop. If the file is not current, press the <b>Delete</b> button to eliminate it.</div>'),
        );
        $form['export']['export_file'] = array(
            '#type' => 'textfield',
            '#title' => t('Latest Export File'),
            '#attributes' => array('value' => $latest_export_file, 'readonly' => 'readonly'),
            '#prefix' => '<table><tr><td width=450px>',
            '#suffix' => '</td>',
        );
        $form['export']['delete'] = array(
            '#type' => 'submit',
            '#value' => t('Delete'),
            '#submit' => array('mercury_utilites_delete_export_file'),
            '#prefix' => '<td align=left style="padding-top:13px">',
            '#suffix' => '</td></tr></table><script>
             	jQuery("#edit-delete").live("click", function() {
  							jQuery("#edit-delete").css("visibility", "hidden");
             	});
  					</script>',
        );
        $form['export']['download'] = array(
            '#type' => 'submit',
            '#value' => t('Export Configuration'),
            '#submit' => array('mercury_utilites_download_export_file'),
        );
    } else {
        $form['export']['export_text'] = array(
            '#type' => 'markup',
            '#markup' => t('<div>Click the <b>Generate Export File</b> button to create a CSV file with the current configuration data from this system.</div><br/>'),
        );
        $form['export']['gen_export'] = array(
            '#type' => 'submit',
            '#value' => t('Generate Export File'),
            '#submit' => array('mercury_utilites_generate_export_file'),
        		'#suffix' => '<script>
             	jQuery("#edit-gen-export").live("click", function() {
  							jQuery("#edit-gen-export").css("visibility", "hidden");
             	});
  					</script>'
        );
    }

    $form['import'] = array(
        '#type' => 'fieldset',
        '#title' => 'Import',
        '#collapsible' => false,
        '#collapsed' => false
    );
    $form['import']['import_text'] = array(
        '#type' => 'markup',
        '#markup' => t('<div>Select a configuration data file from your desktop using the <b>Browse</b> button, then click <b>Import Configuration</b> to begin.</div><br/>'),
    );
    $form['import']['auth_csv_file'] = array(
        '#type' => 'file',
        '#title' => t('Select an import file.'),
        '#description' => t('Upload a CSV file to restore configuration, allowed extensions: .csv'),
    );

    // Important: file uploads do not work well over AJAX so use normal POST.
    // Also: for a multi-button form, the submits CANNOT be disabled via JS.
    $form['import']['submit'] = array(
        '#type' => 'submit',
        '#value' => t('Import Configuration'),
        '#submit' => array('mercury_utilites_upload_import_file'),
    		'#suffix' => '<script>
             	jQuery("#edit-submit").live("click", function() {
  							jQuery("#edit-submit").css("visibility", "hidden");
             	});
  					</script>'
    );

    return $form;
}

function mercury_utilites_delete_export_file($form, &$form_state) {
    $values = $form_state['values'];
    $export_file = $values['export_file'];
    unlink(EXPORT_IMPORT_PATH . $export_file);
    mercury_utilities_fmc_log_message('System Settings', t("Export file <b>$export_file</b> has been deleted."), WATCHDOG_INFO, 'show');
    drupal_goto($form['#action']);
}

function mercury_utilites_generate_export_file($form, &$form_state) {
  /*
   * First make sure there isn't a file already.
   * The interface only supports one file at this time,
   * and it's confusing when there's more than one file in the directory.
   */
  $latest_export_file = mercury_utilities_get_latestFile(EXPORT_IMPORT_PATH);
  if ( $latest_export_file && $latest_export_file != 'not_found' ) {
    mercury_utilities_fmc_set_message(t("Export file <b>$latest_export_file</b> already exists. Please delete the existing file before creating a new one."), 'warning', 'show');
    return drupal_goto($form['#action']);
  }

  // Gather all required data.
  $data = MercuryImportExport::export_gather();

  // No data = gathering failed.
  if (empty($data)) {
    mercury_utilities_fmc_log_message('System Settings', t("No data available to export."), WATCHDOG_ERROR, 'show');
    drupal_goto($form['#action']);
  }

  // Initialize a CSV file for writing.
  @mkdir(EXPORT_IMPORT_PATH, 0777, true);
  $export_file = 'FAMC-Export-Data_' . date('Y-m-d_H:i:s') . '.csv';
  $fh = fopen(EXPORT_IMPORT_PATH . $export_file, 'w');

  // Write the header line to the file.
  fputcsv($fh, MercuryImportExport::$CSV_field_map);

  // Write all data to the file.
  foreach ($data as $row) {
    fputcsv($fh, MercuryImportExport::export_normalize($row));
  }

  // All data written!  Clean up and send a response.
  fclose($fh);
  mercury_utilities_fmc_log_message('System Settings', t("Export file <b>$export_file</b> has been created."), WATCHDOG_INFO, 'show');
  drupal_goto($form['#action']);
}

function mercury_utilites_download_export_file($form, &$form_state) {
    $values = $form_state['values'];
    $export_file = $values['export_file'];
    mercury_utilities_fmc_log_message('System Settings',t("Import file <b>$export_file</b> has been downloaded."));
    drupal_goto('http://' . $_SERVER['HTTP_HOST'] .  EXPORT_IMPORT_BROWSER_PATH . $export_file);
}

function mercury_utilites_upload_import_file($form, &$form_state) {
  $file_path = $_FILES['files']['tmp_name']['auth_csv_file'];

  if (empty($file_path) || ($fh = fopen($file_path, "r")) === FALSE) {
    mercury_utilities_fmc_log_message('System Settings', t("CSV file could not be opened for reading."), WATCHDOG_ERROR, 'show');
    drupal_goto($form['#action']);
  }

  $modified_count = 0;
  try {
    while (($row = fgetcsv($fh)) !== FALSE) {
      $modified_count += MercuryImportExport::import_row($row);
    }
  } catch (Exception $e) {
    mercury_utilities_fmc_log_message(
      'System Settings',
      t('Import failed. Reason: @reason', array('@reason' => $e->getMessage())),
      WATCHDOG_ERROR, 'show'
    );
    drupal_goto($form['#action']);
  }

  mercury_utilities_fmc_log_message('System Settings', t("CSV data has been processed. $modified_count credentials modified."), WATCHDOG_INFO, 'show');
  drupal_goto($form['#action']);
}
